import re
from unittest import skip, skipIf, skipUnless

from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.test.utils import override_settings

from cms.api import add_plugin
from cms.models import Placeholder
from cms.utils import get_current_site
from cms.utils.i18n import force_language
from cms.utils.plugins import downcast_plugins
from cms.utils.urlutils import add_url_parameters, admin_reverse

from djangocms_alias.constants import (
    CATEGORY_SELECT2_URL_NAME,
    DELETE_ALIAS_URL_NAME,
    LIST_ALIAS_URL_NAME,
    SELECT2_ALIAS_URL_NAME,
    USAGE_ALIAS_URL_NAME,
)
from djangocms_alias.models import Alias, AliasContent, Category
from djangocms_alias.utils import is_versioning_enabled

from .base import BaseAliasPluginTestCase


class AliasViewsTestCase(BaseAliasPluginTestCase):

    def test_create_alias_view_get_no_data(self):
        with self.login_user_context(self.superuser):
            response = self.client.get(self.get_create_alias_endpoint())
            self.assertEqual(response.status_code, 400)

    def test_create_alias_view_non_staff_denied_access(self):
        response = self.client.get(self.get_create_alias_endpoint())
        self.assertEqual(response.status_code, 403)

    def test_create_alias_view_get_show_form_plugin(self):
        with self.login_user_context(self.superuser):
            response = self.client.get(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)
            self.assertEqual(
                response.context['form'].initial['plugin'].pk,
                self.plugin.pk,
            )

    def test_create_alias_view_get_show_form_placeholder(self):
        with self.login_user_context(self.superuser):
            response = self.client.get(self.get_create_alias_endpoint(), data={
                'placeholder': self.placeholder.pk,
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)
            self.assertEqual(
                response.context['form'].initial['placeholder'],
                self.placeholder,
            )

    def test_create_alias_view_show_form_replace_hidden(self):
        user = self.get_staff_user_with_std_permissions()
        user.user_permissions.add(
            Permission.objects.get(
                content_type=ContentType.objects.get_for_model(
                    Alias,
                ),
                codename='add_alias',
            )
        )
        with self.login_user_context(user):
            response = self.client.get(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)
            self.assertTrue(
                response.context['form'].fields['replace'].widget.is_hidden,
            )

    def test_create_alias_view_post_plugin(self):
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'category': self.category.pk,
                'language': self.language,
                'name': 'test alias',
            })
            self.assertEqual(response.status_code, 200)

        alias = Alias.objects.last()
        if is_versioning_enabled():
            self._publish(alias)
        alias_plugins = alias.get_placeholder(self.language).get_plugins()

        # Source plugin is kept in original placeholder
        self.assertIn(
            self.plugin,
            downcast_plugins(self.placeholder.get_plugins()),
        )

        self.assertEqual(alias_plugins.count(), 1)
        self.assertEqual(alias_plugins[0].plugin_type, self.plugin.plugin_type)
        self.assertEqual(
            alias_plugins[0].get_bound_plugin().body,
            self.plugin.body,
        )

    def test_create_alias_view_post_plugin_replace(self):
        placeholder = self.placeholder
        if is_versioning_enabled():
            # Can only edit page/content that is in DRAFT
            placeholder = self._get_draft_page_placeholder()

        plugin = add_plugin(
            placeholder,
            'TextPlugin',
            language='en',
            body='test 222',
        )

        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': plugin.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
                'replace': True,
            })

        self.assertEqual(response.status_code, 200)

        alias = Alias.objects.last()
        if is_versioning_enabled():
            self._publish(alias)
        plugins = alias.get_placeholder(self.language).get_plugins()

        self.assertEqual(plugins.count(), 1)
        self.assertEqual(plugins[0].plugin_type, self.plugin.plugin_type)
        self.assertEqual(
            plugins[0].get_bound_plugin().body,
            plugin.body,
        )

    def test_create_alias_view_name(self):
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)

        alias = Alias.objects.last()
        if is_versioning_enabled():
            self._publish(alias)
        self.assertEqual(alias.name, 'test alias')

    def test_create_alias_view_post_no_plugin_or_placeholder(self):
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)
            self.assertFalse(response.context['form'].is_valid())

    def test_create_alias_view_post_both_plugin_and_placeholder(self):
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'placeholder': self.placeholder.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)
            self.assertFalse(response.context['form'].is_valid())

    def test_create_alias_view_post_empty_placeholder(self):
        placeholder = Placeholder(slot='empty')
        placeholder.save()

        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'placeholder': placeholder.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })
            self.assertEqual(response.status_code, 400)
            self.assertEqual(
                response.content.decode(),
                'Plugins are required to create an alias',
            )

    def test_create_alias_view_post_placeholder(self):
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'placeholder': self.placeholder.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)

        # Source plugins are kept in original placeholder
        plugins = self.placeholder.get_plugins()
        self.assertEqual(plugins.count(), 1)
        plugin_in_placeholder = plugins[0].get_bound_plugin()
        self.assertEqual(self.plugin, plugin_in_placeholder)

        alias = Alias.objects.first()
        if is_versioning_enabled():
            self._publish(alias)

        source_plugins = self.placeholder.get_plugins()
        alias_plugins = alias.get_placeholder(self.language).get_plugins()

        self.assertEqual(alias_plugins.count(), source_plugins.count())
        for source, target in zip(source_plugins, alias_plugins):
            self.assertEqual(source.plugin_type, target.plugin_type)
            self.assertEqual(
                source.get_bound_plugin().body,
                target.get_bound_plugin().body,
            )

    def test_create_alias_view_post_placeholder_replace(self):
        placeholder = self.placeholder
        if is_versioning_enabled():
            # Can only edit page/content that is in DRAFT
            placeholder = self._get_draft_page_placeholder()

        placeholder.get_plugins().delete()
        text_plugin = add_plugin(
            placeholder,
            'TextPlugin',
            language='en',
            body='test 2',
        )

        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'placeholder': placeholder.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
                'replace': True,
            })
            self.assertEqual(response.status_code, 200)

        alias = Alias.objects.last()
        if is_versioning_enabled():
            self._publish(alias)
        alias_plugins = alias.get_placeholder(self.language).get_plugins()

        self.assertEqual(alias_plugins.count(), 1)
        self.assertEqual(alias_plugins[0].plugin_type, self.plugin.plugin_type)
        self.assertEqual(
            alias_plugins[0].get_bound_plugin().body,
            text_plugin.body,
        )

        placeholder_plugins = placeholder.get_plugins()
        self.assertEqual(placeholder_plugins.count(), 1)

        self.assertEqual(
            placeholder_plugins[0].get_bound_plugin().alias,
            alias,
        )

    def test_create_alias_view_post_no_create_permission(self):
        with self.login_user_context(self.get_staff_user_with_no_permissions()):  # noqa: E501
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })
            self.assertEqual(response.status_code, 403)

    def test_create_alias_view_post_no_replace_permission(self):
        user = self.get_staff_user_with_std_permissions()
        user.user_permissions.add(
            Permission.objects.get(
                content_type=ContentType.objects.get_for_model(
                    Alias,
                ),
                codename='add_alias',
            )
        )
        with self.login_user_context(user):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
                'replace': True,
            })
            self.assertEqual(response.status_code, 403)

    def test_detach_view_no_permission_to_add_plugins_from_alias(self):
        response = self.client.post(
            self.get_detach_alias_plugin_endpoint(self.plugin.pk),
        )
        self.assertEqual(response.status_code, 403)

    def test_detach_view_get(self):
        alias = self._create_alias([self.plugin])
        plugin = add_plugin(
            self.placeholder,
            'Alias',
            language='en',
            alias=alias,
        )
        with self.login_user_context(self.superuser):
            response = self.client.get(
                self.get_detach_alias_plugin_endpoint(plugin.pk),
            )
            self.assertEqual(response.status_code, 200)

    def test_detach_view_non_staff_denied_access(self):
        alias = self._create_alias([self.plugin])
        plugin = add_plugin(
            self.placeholder,
            'Alias',
            language='en',
            alias=alias,
        )
        user = self.get_staff_user_with_no_permissions()
        with self.login_user_context(user):
            response = self.client.post(
                self.get_detach_alias_plugin_endpoint(plugin.pk),
            )
        self.assertEqual(response.status_code, 403)

    def test_detach_view_non_alias_plugin(self):
        with self.login_user_context(self.superuser):
            response = self.client.post(
                self.get_detach_alias_plugin_endpoint(self.plugin.pk),
            )
            self.assertEqual(response.status_code, 404)

    def test_detach_view(self):
        placeholder = self.placeholder
        if is_versioning_enabled():
            # Can only edit page/content that is in DRAFT
            placeholder = self._get_draft_page_placeholder()

        placeholder.get_plugins().delete()
        alias = self._create_alias()
        add_plugin(
            placeholder,
            'TextPlugin',
            language=self.language,
            body='test',
        )
        plugin = add_plugin(
            placeholder,
            'Alias',
            language='en',
            alias=alias,
        )
        add_plugin(
            alias.get_placeholder(self.language),
            'TextPlugin',
            language=self.language,
            body='test 2',
        )
        add_plugin(
            alias.get_placeholder(self.language),
            'TextPlugin',
            language=self.language,
            body='test 88',
        )
        plugins = placeholder.get_plugins()
        self.assertEqual(plugins.count(), 2)

        with self.login_user_context(self.superuser):
            response = self.client.post(
                self.get_detach_alias_plugin_endpoint(plugin.pk),
            )
            self.assertEqual(response.status_code, 200)

        plugins = placeholder.get_plugins()
        self.assertEqual(plugins.count(), 3)
        self.assertEqual(plugins[0].get_bound_plugin().body, 'test')
        self.assertEqual(plugins[1].get_bound_plugin().body, 'test 2')
        self.assertEqual(plugins[2].get_bound_plugin().body, 'test 88')

    def test_alias_content_preview_view(self):
        alias = self._create_alias([self.plugin])
        with self.login_user_context(self.superuser):
            response = self.client.get(alias.get_content().get_absolute_url())

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, alias.name)
        self.assertContains(response, self.plugin.body)

    def test_view_aliases_using_site_filter(self):
        site1 = Site.objects.create(domain='site1.com', name='1')
        site2 = Site.objects.create(domain='site2.com', name='2')
        site1_plugin = add_plugin(
            self.placeholder,
            'TextPlugin',
            language='en',
            body='This is text in English',
        )
        site2_plugin = add_plugin(
            self.placeholder,
            'TextPlugin',
            language='en',
            body='Das ist Text auf Deutsch',
        )
        site1_alias = self._create_alias(plugins=[site1_plugin], site=site1, name='site1_alias', category=self.category)
        site2_alias = self._create_alias(plugins=[site2_plugin], site=site2, name='site2_alias', category=self.category)
        alias_list_url = admin_reverse(LIST_ALIAS_URL_NAME)

        # when no filter used both objects are displayed
        with self.login_user_context(self.superuser):
            with force_language('en'):
                list_response = self.client.get(alias_list_url)

        self.assertContains(list_response, site1_alias.name)
        self.assertContains(list_response, site2_alias.name)

        # when no filtering by site 1 only first object displayed
        with self.login_user_context(self.superuser):
            with force_language('en'):
                site1_aliases_filter_url = f"{alias_list_url}?site={site1_alias.site.id}"
                list_response = self.client.get(site1_aliases_filter_url)

        self.assertContains(list_response, site1_alias.name)
        self.assertNotContains(list_response, site2_alias.name)

        # when no filtering by site 2 only first object displayed
        with self.login_user_context(self.superuser):
            with force_language('en'):
                site2_aliases_filter_url = f"{alias_list_url}?site={site2_alias.site.id}"
                list_response = self.client.get(site2_aliases_filter_url)

        self.assertNotContains(list_response, site1_alias.name)
        self.assertContains(list_response, site2_alias.name)

    @skipIf(is_versioning_enabled(), 'Test only relevant without versioning enabled')
    def test_create_alias_name_unique_per_category_and_language(self):
        self._create_alias(
            name='test alias',
            category=self.category,
        )
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })

        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'Alias with this Name and Category already exists.',
        )
        self.assertEqual(
            AliasContent.objects.filter(
                name='test alias',
                language=self.language,
                alias__category=self.category,
            ).count(),
            1,
        )

    def test_select2_view_no_permission(self):
        response = self.client.get(
            admin_reverse(
                SELECT2_ALIAS_URL_NAME,
            ),
        )
        self.assertEqual(response.status_code, 403)

    def test_select2_view(self):
        alias1 = self._create_alias(name='test 2')
        alias2 = self._create_alias(name='foo', position=1)
        alias3 = self._create_alias(name='foo4', position=1, published=False)

        if is_versioning_enabled():
            from djangocms_versioning.constants import DRAFT
            from djangocms_versioning.models import Version

            # This will show because it's a new draft version of the same alias
            draft_content = alias2.contents.create(name='foo', language=self.language)
            Version.objects.create(
                content=draft_content, created_by=self.superuser, state=DRAFT)

        # This shouldn't show because it hasn't content in current language
        self._create_alias(name='foo2', language='fr', position=1)
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
            )

        result = [alias1.pk, alias2.pk, alias3.pk]
        text_result = ['test 2']

        if is_versioning_enabled():
            # The following versions have draft content
            text_result.append('foo (Not published)')
            text_result.append('foo4 (Not published)')
        else:
            text_result.append('foo')
            text_result.append('foo4')

        self.assertEqual(response.status_code, 200)
        self.assertEqual([a['id'] for a in response.json()['results']], result)
        self.assertEqual(
            [a['text'] for a in response.json()['results']],
            text_result,
        )

    def test_select2_view_order_by_category_and_position(self):
        category2 = Category.objects.create(name='foo')
        alias1 = self._create_alias(name='test 2')
        alias2 = self._create_alias(name='foo', position=1)
        alias3 = self._create_alias(name='bar', category=category2)
        alias4 = self._create_alias(name='baz', category=category2, position=1)
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            [a['id'] for a in response.json()['results']],
            [alias3.pk, alias4.pk, alias1.pk, alias2.pk],
        )

    def test_select2_view_set_limit(self):
        self._create_alias(name='test 2')
        self._create_alias(name='foo', position=1)
        self._create_alias(name='three', position=2)
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
                data={'limit': 2},
            )

        content = response.json()

        self.assertEqual(response.status_code, 200)
        self.assertTrue(content['more'])
        self.assertEqual(len(content['results']), 2)

    def test_select2_view_text_repr(self):
        alias1 = self._create_alias(name='test 2')
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.json()['results'][0]['text'],
            alias1.name,
        )

    def test_select2_view_term(self):
        alias1 = self._create_alias(name='test 2')
        self._create_alias(name='foo', position=1)
        alias3 = self._create_alias(name='three', position=2)
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
                data={'term': 't'},
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            [a['id'] for a in response.json()['results']],
            [alias1.pk, alias3.pk],
        )

    def test_select2_view_category(self):
        category2 = Category.objects.create(name='test 2')
        alias1 = self._create_alias(name='test 2', category=category2)
        alias2 = self._create_alias(name='foo', category=category2, position=1)
        # This shouldnt show because it's in different Category
        self._create_alias(name='three')
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
                data={'category': category2.pk},
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            [a['id'] for a in response.json()['results']],
            [alias1.pk, alias2.pk],
        )

    def test_select2_view_category_and_term(self):
        category2 = Category.objects.create(name='test 2')
        alias1 = self._create_alias(name='test 2', category=category2)
        self._create_alias(name='foo', category=category2, position=1)
        self._create_alias(name='three')
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
                data={'category': category2.pk, 'term': 't'},
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            [a['id'] for a in response.json()['results']],
            [alias1.pk],
        )

    def test_select2_view_site(self):
        """
        The list is filtered based on only matching
        alias with a specific site if it is provided
        """
        site = get_current_site()
        alias1 = self._create_alias(site=site)
        alias2 = self._create_alias(site=site, position=1)
        alias3 = self._create_alias(position=2)

        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
                data={'site': site.pk},
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            [a['id'] for a in response.json()['results']],
            [alias1.pk, alias2.pk, alias3.pk],
        )

    def test_select2_view_site_and_category(self):
        """
        The list is filtered based on only matching
        alias with a specific site and category if it is provided
        """
        category = Category.objects.create(name='category')
        site = get_current_site()
        self._create_alias(site=site)
        alias2 = self._create_alias(site=site, category=category, position=1)
        self._create_alias(position=2)
        self._create_alias(position=3)

        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
                data={
                    'site': site.pk,
                    'category': category.pk,
                },
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            [a['id'] for a in response.json()['results']],
            [alias2.pk],
        )

    def test_select2_view_pk(self):
        alias1 = self._create_alias(name='test 2')
        self._create_alias(name='foo', position=1)
        self._create_alias(name='three')
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    SELECT2_ALIAS_URL_NAME,
                ),
                data={'pk': alias1.pk},
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            [a['id'] for a in response.json()['results']],
            [alias1.pk],
        )

    @skip(
        "It is not currently possible to add an alias from the django admin changelist issue "
        "#https://github.com/django-cms/djangocms-alias/issues/97#97"
    )
    def test_aliascontent_add_view(self):
        alias = Alias.objects.create(category=self.category)
        with self.login_user_context(self.superuser):
            response = self.client.post(
                admin_reverse('djangocms_alias_aliascontent_add'),
                data={
                    'language': 'de',
                    'name': 'alias test de 1',
                    'alias': alias.pk,
                },
            )

        self.assertEqual(response.status_code, 302)
        if is_versioning_enabled():
            self._publish(alias, 'de')
        self.assertEqual(alias.contents.count(), 1)
        alias_content = alias.contents.first()
        self.assertEqual(alias_content.language, 'de')
        self.assertEqual(alias_content.name, 'alias test de 1')

    @skip(
        "It is not currently possible to add an alias from the django admin changelist issue "
        "#https://github.com/django-cms/djangocms-alias/issues/97#97"
    )
    def test_aliascontent_add_view_get(self):
        alias = Alias.objects.create(category=self.category)
        with self.login_user_context(self.superuser):
            response = self.client.get(
                add_url_parameters(
                    admin_reverse(
                        'djangocms_alias_aliascontent_add',
                    ),
                    language='fr',
                    alias=alias.pk,
                )
            )

        self.assertContains(response, 'type="hidden" name="language" value="fr"')
        self.assertContains(response, f'type="hidden" name="alias" value="{alias.pk}"')

    @skip(
        "It is not currently possible to add an alias from the django admin changelist issue "
        "#https://github.com/django-cms/djangocms-alias/issues/97#97"
    )
    def test_aliascontent_add_view_invalid_data(self):
        alias = Alias.objects.create(category=self.category)
        self._create_alias(
            name='test alias',
            category=self.category,
            language=self.language,
            published=True,
        )
        with self.login_user_context(self.superuser):
            response = self.client.post(
                admin_reverse('djangocms_alias_aliascontent_add'),
                data={
                    'language': self.language,
                    'name': 'test alias',
                    'alias': alias.pk,
                },
            )

        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'Alias with this Name and Category already exists',
        )

    @skip(
        "It is not currently possible to add an alias from the django admin changelist issue "
        "#https://github.com/django-cms/djangocms-alias/issues/97#97"
    )
    def test_aliascontent_add_view_valid_data(self):
        alias = Alias.objects.create(category=self.category)
        if is_versioning_enabled():
            self._create_alias(
                name='test alias',
                category=self.category,
                language=self.language,
                published=False,
            )
        with self.login_user_context(self.superuser):
            response = self.client.post(
                admin_reverse('djangocms_alias_aliascontent_add'),
                data={
                    'language': self.language,
                    'name': 'test alias',
                    'alias': alias.pk,
                },
            )

        self.assertEqual(response.status_code, 302)
        if is_versioning_enabled():
            self._publish(alias)
        alias_content = alias.contents.first()
        self.assertEqual(alias_content.name, 'test alias')

    def test_alias_usage_view(self):
        alias = self._create_alias()
        root_alias = self._create_alias()
        self.add_alias_plugin_to_page(self.page, alias)
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    USAGE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
            )

        self.assertContains(response, '<td>Page</td>')
        self.assertNotContains(response, '<td>Alias</td>')

        add_plugin(
            root_alias.get_placeholder(self.language),
            'Alias',
            language=self.language,
            alias=alias,
        )

        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    USAGE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
            )

        self.assertContains(response, '<td>Page</td>')
        self.assertContains(response, '<td>Alias</td>')
        self.assertRegex(
            str(response.content),
            r'href="{}"[\w+]?>{}<\/a>'.format(
                re.escape(self.page.get_absolute_url(self.language)),
                str(self.page),
            ),
        )
        self.assertRegex(
            str(response.content),
            r'href="{}"[\w+]?>{}<\/a>'.format(
                re.escape(root_alias.get_absolute_url()),
                str(alias),
            ),
        )
        self.assertRegex(
            str(response.content),
            r'href="{}"[\w+]?>{}<\/a>'.format(
                re.escape(
                    add_url_parameters(
                        admin_reverse(
                            USAGE_ALIAS_URL_NAME,
                            args=[root_alias.pk],
                        ),
                        back=1,
                    )
                ),
                'View usage',
            ),
        )

    def test_delete_alias_view_get(self):
        alias = self._create_alias([self.plugin])
        add_plugin(
            self.placeholder,
            'Alias',
            language=self.language,
            alias=alias,
        )
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
            )
        self.assertContains(
            response,
            f'Are you sure you want to delete the alias "{alias.name}"?',  # noqa: E501
        )

    def test_delete_alias_view_get_using_objects(self):
        alias = self._create_alias([self.plugin])
        add_plugin(
            self.placeholder,
            'Alias',
            language=self.language,
            alias=alias,
        )
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
            )
        self.assertContains(response, 'This alias is used by following objects:')
        test = r'<li>[\s\\n]*Page:[\s\\n]*<a href=\"\/en\/test\/\">test<\/a>[\s\\n]*<\/li>'
        self.assertRegex(str(response.content), test)

    def test_delete_alias_view_get_alias_not_used_on_any_page(self):
        alias = self._create_alias([self.plugin])
        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
            )
        self.assertContains(response, 'This alias wasn\'t used by any object.')

    def test_delete_alias_view_post(self):
        """Tests the admin delete view (as opposed to the djangocms_alias.views.delete_view)"""
        alias = self._create_alias([self.plugin])
        self.assertIn(alias, Alias.objects.all())
        with self.login_user_context(self.superuser):
            response = self.client.post(
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
                data={'post': 'yes'},
            )
        self.assertEqual(response.status_code, 302)  # Successful delete returns a redirect
        self.assertFalse(Alias.objects.filter(pk=alias.pk).exists())  # Ensure it's gone

    def test_delete_alias_view_user_with_no_perms(self):
        alias = self._create_alias([self.plugin])
        staff_user = self.get_staff_user_with_no_permissions()
        with self.login_user_context(staff_user):  # noqa: E501
            response = self.client.get(
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
            )
        self.assertEqual(response.status_code, 403)

        with self.login_user_context(staff_user):  # noqa: E501
            response = self.client.post(
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
                data={'post': 'yes'},
            )
        self.assertEqual(response.status_code, 403)

    def test_delete_alias_view_alias_not_being_used(self):
        alias = self._create_alias([self.plugin])
        staff_user = self.get_staff_user_with_alias_permissions()
        with self.login_user_context(staff_user):  # noqa: E501
            response = self.client.get(
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
            )
        self.assertContains(response, 'Are you sure you want to delete')
        self.assertContains(response, 'type="submit"')
        with self.login_user_context(staff_user):  # noqa: E501
            response = self.client.post(  # noqa
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
                data={'post': 'yes'},
            )
        self.assertEqual(response.status_code, 302)
        self.assertFalse(Alias.objects.filter(pk=alias.pk).exists())

    def test_delete_alias_view_alias_being_used_on_pages(self):
        alias = self._create_alias([self.plugin])
        add_plugin(
            self.placeholder,
            'Alias',
            language=self.language,
            alias=alias,
        )
        # this user only can delete alias when alias not being used anywhere
        staff_user = self.get_staff_user_with_no_permissions()
        self.add_permission(staff_user, 'delete_alias')

        with self.login_user_context(staff_user):  # noqa: E501
            response = self.client.get(
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
            )
        self.assertEqual(response.status_code, 403)

        with self.login_user_context(staff_user):
            response = self.client.post(
                admin_reverse(
                    DELETE_ALIAS_URL_NAME,
                    args=[alias.pk],
                ),
                data={'post': 'yes'},
            )
        self.assertEqual(response.status_code, 403)

    def test_custom_template_alias_view(self):
        alias = self._create_alias()
        add_plugin(
            alias.get_placeholder(self.language),
            'TextPlugin',
            language=self.language,
            body='custom alias content',
        )
        add_plugin(
            self.placeholder,
            'Alias',
            language=self.language,
            alias=alias,
            template='custom_alias_template',
        )

        response = self.client.get(self.page.get_absolute_url())
        self.assertContains(response, '<b>custom alias content</b>')

    @override_settings(CMS_PLACEHOLDER_CACHE=False)
    def test_alias_multisite_support(self):
        site1 = Site.objects.create(domain='site1.com', name='1')
        site2 = Site.objects.create(domain='site2.com', name='2')
        alias = self._create_alias()
        alias_placeholder = alias.get_placeholder(self.language)
        add_plugin(
            alias_placeholder,
            'TextPlugin',
            language=self.language,
            body='test alias multisite',
        )

        site1_page = self._create_page(
            title='Site1',
            language=self.language,
            site=site1,
        )
        site2_page = self._create_page(
            title='Site2',
            language=self.language,
            site=site2,
        )
        self.add_alias_plugin_to_page(site1_page, alias)
        self.add_alias_plugin_to_page(site2_page, alias)

        with override_settings(SITE_ID=site1.pk):
            response = self.client.get(site1_page.get_absolute_url())
        self.assertContains(response, 'test alias multisite')

        with override_settings(SITE_ID=site2.pk):
            response = self.client.get(site2_page.get_absolute_url())
        self.assertContains(response, 'test alias multisite')

        add_plugin(
            alias_placeholder,
            'TextPlugin',
            language=self.language,
            body='Another alias plugin',
        )

        with override_settings(SITE_ID=site1.pk):
            response = self.client.get(site1_page.get_absolute_url())
        self.assertContains(response, 'test alias multisite')
        self.assertContains(response, 'Another alias plugin')

        with override_settings(SITE_ID=site2.pk):
            response = self.client.get(site2_page.get_absolute_url())
        self.assertContains(response, 'test alias multisite')
        self.assertContains(response, 'Another alias plugin')


class AliasCategorySelect2ViewTestCase(BaseAliasPluginTestCase):

    def test_select2_view_no_permission(self):
        """
        The category list view is private
        and only intended for use in the admin
        """
        response = self.client.get(
            admin_reverse(
                CATEGORY_SELECT2_URL_NAME,
            ),
        )
        self.assertEqual(response.status_code, 403)

    def test_select2_view_alias_not_set(self):
        """
        When categories exist but are not attached to an alias
        they are ignored
        """
        Category.objects.create(name='Category 1')
        category_2 = Category.objects.create(name='Category 2')
        Category.objects.create(name='Category 3')
        self._create_alias(category=category_2)

        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    CATEGORY_SELECT2_URL_NAME,
                ),
            )

        expected_result = [category_2.pk]

        self.assertEqual(response.status_code, 200)
        self.assertEqual([a['id'] for a in response.json()['results']], expected_result)

    def test_select2_view_alias_site_set(self):
        """
        When a site is supplied only categories with site entries
        are returned
        """
        site = get_current_site()
        second_site = Site.objects.create(domain="other-site.org", name="other site")
        category_1 = Category.objects.create(name='Category 1')
        category_2 = Category.objects.create(name='Category 2')
        category_3 = Category.objects.create(name='Category 3')
        self._create_alias(category=category_1, site=site)
        self._create_alias(category=category_2, site=second_site)
        self._create_alias(category=category_3)

        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    CATEGORY_SELECT2_URL_NAME,
                ),
                data={'site': site.pk},
            )

        expected_result = [category_1.pk, category_3.pk]

        self.assertEqual(response.status_code, 200)
        self.assertEqual([a['id'] for a in response.json()['results']], expected_result)

    def test_select2_view_set_limit(self):
        """
        Ensure that the page limit is respected
        """
        category_1 = Category.objects.create(name='Category 1')
        category_2 = Category.objects.create(name='Category 2')
        category_3 = Category.objects.create(name='Category 3')
        self._create_alias(category=category_1)
        self._create_alias(category=category_2, position=1)
        self._create_alias(category=category_3, position=2)

        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    CATEGORY_SELECT2_URL_NAME,
                ),
                data={'limit': 2},
            )

        content = response.json()

        self.assertEqual(response.status_code, 200)
        self.assertTrue(content['more'])
        self.assertEqual(len(content['results']), 2)

    def test_select2_view_text_repr(self):
        """
        Ensure that the display / text representation of the object
        is output to the user.
        """
        category = Category.objects.create(name='Category 1')
        self._create_alias(name='test 2', category=category)

        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    CATEGORY_SELECT2_URL_NAME,
                ),
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.json()['results'][0]['text'],
            category.name,
        )

    def test_select2_view_term(self):
        """
        Given a term, the response should return only
        categories that match the term.
        """
        category_1 = Category.objects.create(name='ategory 1')
        category_2 = Category.objects.create(name='Category 2')
        category_3 = Category.objects.create(name='tegory 3')
        category_4 = Category.objects.create(name='tegory 4')
        self._create_alias(category=category_1)
        self._create_alias(category=category_2)
        self._create_alias(category=category_3)
        self._create_alias(category=category_4)

        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    CATEGORY_SELECT2_URL_NAME,
                ),
                data={'term': 'ate'},
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            [a['id'] for a in response.json()['results']],
            [category_2.pk, category_1.pk],
        )

    def test_select2_view_pk(self):
        """
        When a pk is provided that record should be returned
        """
        category_1 = Category.objects.create(name='Category 1')
        category_2 = Category.objects.create(name='Category 2')
        category_3 = Category.objects.create(name='Category 3')
        self._create_alias(category=category_1)
        self._create_alias(category=category_2)
        self._create_alias(category=category_3)

        with self.login_user_context(self.superuser):
            response = self.client.get(
                admin_reverse(
                    CATEGORY_SELECT2_URL_NAME,
                ),
                data={'pk': category_2.pk},
            )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            [a['id'] for a in response.json()['results']],
            [category_2.pk],
        )


class AliasViewsUsingVersioningTestCase(BaseAliasPluginTestCase):

    @skipUnless(is_versioning_enabled(), 'Test only relevant for versioning')
    def test_create_alias_view_name_draft_alias(self):
        with self.login_user_context(self.superuser):
            name = 'test alias'
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'category': self.category.pk,
                'name': name,
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)

        alias = Alias.objects.last()
        # AliasContent not published
        self.assertEqual(alias.name, f'{name} (Not published)')

    @skipUnless(is_versioning_enabled(), 'Test only relevant for versioning')
    def test_create_alias_view_creating_version(self):
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)

        alias = Alias.objects.last()
        if is_versioning_enabled():
            from djangocms_versioning.models import Version
            self.assertEqual(Version.objects.filter_by_grouper(alias).count(), 1)

    @skipUnless(is_versioning_enabled(), 'Test only relevant for versioning')
    def test_create_alias_name_without_uniqness(self):
        alias1 = self._create_alias(
            name='test alias',
            category=self.category,
            published=True,
        )
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })

        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'Alias with this Name and Category already exists.',
        )

        self._unpublish(alias1)

        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'plugin': self.plugin.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })

        self.assertEqual(response.status_code, 200)
        self.assertNotContains(
            response,
            'Alias with this Name and Category already exists.',
        )

        alias = Alias.objects.last()
        self._publish(alias)
        qs = AliasContent.objects.filter(
            name='test alias',
            language=self.language,
            alias__category=self.category,
        )
        self.assertEqual(qs.count(), 1)
        self.assertEqual(qs.first(), alias.get_content(self.language))

    @skipUnless(is_versioning_enabled(), 'Test only relevant for versioning')
    def test_create_alias_view_post_placeholder_from_draft_page(self):
        page1 = self._create_page('test alias page')
        placeholder = page1.get_placeholders(self.language).get(
            slot='content',
        )
        plugin = add_plugin(
            placeholder,
            'TextPlugin',
            language=self.language,
            body='test alias',
        )
        self._unpublish(page1)
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'placeholder': placeholder.pk,
                'category': self.category.pk,
                'name': 'test alias',
                'language': self.language,
            })
            self.assertEqual(response.status_code, 200)

        # Source plugins are kept in original placeholder
        plugins = placeholder.get_plugins()
        self.assertEqual(plugins.count(), 1)
        plugin_in_placeholder = plugins[0].get_bound_plugin()
        self.assertEqual(plugin, plugin_in_placeholder)

        alias = Alias.objects.first()
        self._publish(alias)

        source_plugins = placeholder.get_plugins()
        alias_plugins = alias.get_placeholder(self.language).get_plugins()

        self.assertEqual(alias_plugins.count(), source_plugins.count())
        for source, target in zip(source_plugins, alias_plugins):
            self.assertEqual(source.plugin_type, target.plugin_type)
            self.assertEqual(
                source.get_bound_plugin().body,
                target.get_bound_plugin().body,
            )

    @skipUnless(is_versioning_enabled(), 'Test only relevant for versioning')
    def test_create_alias_with_replace_plugin_with_versioning_checks(self):
        # 403 if you try to edit placeholder of published page
        with self.login_user_context(self.superuser):
            response = self.client.post(self.get_create_alias_endpoint(), data={
                'placeholder': self.placeholder.pk,
                'category': self.category.pk,
                'name': 'test alias 5',
                'language': self.language,
                'replace': True,
            })
            self.assertEqual(response.status_code, 403)

    @skipUnless(is_versioning_enabled(), 'Test only relevant for versioning')
    def test_detach_view_with_versioning_checks(self):
        # 403 when placeholder from non-draft page
        alias = self._create_alias()
        plugin = add_plugin(
            self.placeholder,
            'Alias',
            language='en',
            alias=alias,
        )
        with self.login_user_context(self.superuser):
            response = self.client.post(
                self.get_detach_alias_plugin_endpoint(plugin.pk),
            )
            self.assertEqual(response.status_code, 403)

    @skipUnless(is_versioning_enabled(), 'Only valid in versioning scenario')
    def test_alias_not_shown_when_draft_when_visiting_page(self):
        """
        When visiting a published page with a draft alias the alias
        is not visible
        """
        unpublished_alias = self._create_alias(published=False)
        content = unpublished_alias.contents(manager="admin_manager").filter(language=self.language).first()
        alias_placeholder = content.placeholder

        body = 'unpublished alias'
        add_plugin(
            alias_placeholder,
            'TextPlugin',
            language=self.language,
            body=body
        )

        page = self._create_page(
            title='New page',
            language=self.language,
        )

        self.add_alias_plugin_to_page(page, unpublished_alias)
        response = self.client.get(page.get_absolute_url())
        self.assertNotContains(response, body)

    def test_view_multilanguage(self):
        """
        Views should only display content related to the selected language
        """
        en_plugin = add_plugin(
            self.placeholder,
            'TextPlugin',
            language='en',
            body='This is text in English',
        )
        de_plugin = add_plugin(
            self.placeholder,
            'TextPlugin',
            language='de',
            body='Das ist Text auf Deutsch',
        )
        fr_plugin = add_plugin(
            self.placeholder,
            'TextPlugin',
            language='fr',
            body='C\'est le texte en français',
        )
        alias = self._create_alias([en_plugin], name="English AliasContent object")
        alias_content_de = AliasContent.objects.create(
            alias=alias,
            name='German AliasContent object',
            language='de',
        )
        alias_content_de.populate(plugins=[de_plugin])
        alias_content_fr = AliasContent.objects.create(
            alias=alias,
            name='French AliasContent object',
            language='fr',
        )
        alias_content_fr.populate(plugins=[fr_plugin])

        # when versioning is enabled a Version must be created and published for each language
        if is_versioning_enabled():
            from djangocms_versioning.models import Version
            version_de = Version.objects.create(content=alias_content_de, created_by=self.superuser)
            version_de.publish(user=self.superuser)
            version_fr = Version.objects.create(content=alias_content_fr, created_by=self.superuser)
            version_fr.publish(user=self.superuser)

        with self.login_user_context(self.superuser):
            with force_language('en'):
                if is_versioning_enabled():
                    # we need to call get_absolute_url on the AliasContent object when versioning is enabled,
                    # otherwise we are taken to the version list url
                    detail_response = self.client.get(alias.get_content(language="en").get_absolute_url())
                else:
                    detail_response = self.client.get(alias.get_absolute_url())
                list_response = self.client.get(
                    admin_reverse(LIST_ALIAS_URL_NAME),
                )
        self.assertContains(detail_response, en_plugin.body)
        self.assertContains(list_response, alias.name)
        self.assertNotContains(detail_response, fr_plugin.body)
        self.assertNotContains(list_response, alias_content_fr.name)
        self.assertNotContains(detail_response, de_plugin.body)
        self.assertNotContains(list_response, alias_content_de.name)

        with self.login_user_context(self.superuser):
            with force_language('de'):
                if is_versioning_enabled():
                    # we need to call get_absolute_url on the AliasContent object when versioning is enabled,
                    # otherwise we are taken to the version list url
                    detail_response = self.client.get(alias_content_de.get_absolute_url())
                else:
                    detail_response = self.client.get(alias.get_absolute_url())
                list_response = self.client.get(
                    admin_reverse(LIST_ALIAS_URL_NAME),
                )
        self.assertContains(detail_response, de_plugin.body)
        self.assertContains(list_response, alias_content_de.name)
        self.assertNotContains(detail_response, fr_plugin.body)
        self.assertNotContains(list_response, alias_content_fr.name)
        self.assertNotContains(detail_response, en_plugin.body)
        self.assertNotContains(list_response, alias.name)

        with self.login_user_context(self.superuser):
            with force_language('fr'):
                if is_versioning_enabled():
                    # we need to call get_absolute_url on the AliasContent object when versioning is enabled,
                    # otherwise we are taken to the version list url
                    detail_response = self.client.get(alias_content_fr.get_absolute_url())
                else:
                    detail_response = self.client.get(alias.get_absolute_url())
                list_response = self.client.get(
                    admin_reverse(LIST_ALIAS_URL_NAME),  # noqa: E501
                )

        self.assertContains(detail_response, fr_plugin.body)
        self.assertContains(list_response, alias_content_fr.name)
        self.assertNotContains(detail_response, de_plugin.body)
        self.assertNotContains(list_response, alias_content_de.name)
        self.assertNotContains(detail_response, en_plugin.body)
        self.assertNotContains(list_response, alias.name)



from django.contrib.sites.models import Site
from django.utils import translation

from cms.utils import get_current_site
from cms.wizards.forms import WizardStep2BaseForm, step2_form_factory
from cms.wizards.helpers import get_entries as get_wizard_entires

from djangocms_alias.models import Category
from djangocms_alias.utils import is_versioning_enabled

from .base import BaseAliasPluginTestCase


class WizardsTestCase(BaseAliasPluginTestCase):

    def _get_wizard_instance(self, wizard_name):
        return [
            wizard
            for wizard in get_wizard_entires()
            if wizard.__class__.__name__ == wizard_name
        ][0]

    def _get_form_kwargs(self, data, language=None):
        language = language or self.language
        request = self.get_request('/', language=language)
        request.user = self.superuser
        return {
            'data': data,
            'wizard_language': language,
            'wizard_site': Site.objects.get_current(),
            'wizard_request': request,
        }

    def test_create_alias_wizard_instance(self):
        wizard = self._get_wizard_instance('CreateAliasWizard')
        self.assertEqual(wizard.title, 'New alias')

        self.assertTrue(wizard.user_has_add_permission(self.superuser))
        self.assertTrue(
            wizard.user_has_add_permission(self.get_staff_user_with_alias_permissions()),  # noqa: E501
        )
        self.assertFalse(
            wizard.user_has_add_permission(self.get_staff_user_with_no_permissions()),  # noqa: E501
        )
        self.assertFalse(
            wizard.user_has_add_permission(self.get_standard_user()),
        )

    def test_create_alias_wizard_form(self):
        wizard = self._get_wizard_instance('CreateAliasWizard')
        data = {
            'name': 'Content #1',
            'category': self.category.pk,
            'site': get_current_site().pk
        }
        form_class = step2_form_factory(
            mixin_cls=WizardStep2BaseForm,
            entry_form_class=wizard.form,
        )
        form = form_class(**self._get_form_kwargs(data))

        self.assertTrue(form.is_valid())
        alias = form.save()

        self.assertEqual(form.fields['site'].initial, get_current_site())

        with self.login_user_context(self.superuser):
            response = self.client.get(alias.get_absolute_url())
        self.assertContains(response, data['name'])

        if is_versioning_enabled():
            from djangocms_versioning.models import Version
            self.assertEqual(Version.objects.filter_by_grouper(alias).count(), 1)

    def test_create_alias_wizard_form_with_no_category_fallback_language(self):
        """When creating an Alias via the Wizard an error can occur if the category
        doesn't have a valid translation
        """
        translation.activate('en')
        # A japanese translation that does not have any fallback settings!
        Category.objects.language('ja').create(name='Japanese category')

        wizard = self._get_wizard_instance('CreateAliasWizard')
        data = {
            'name': 'Content #1',
            'category': None,
        }
        form_class = step2_form_factory(
            mixin_cls=WizardStep2BaseForm,
            entry_form_class=wizard.form,
        )
        form = form_class(**self._get_form_kwargs(data))
        category_form_queryset = form.declared_fields['category'].queryset

        self.assertEqual(form.fields['site'].initial, get_current_site())

        # Be sure that we have untranslated categories that enforces a fair test
        self.assertNotEqual(
            category_form_queryset.all().count(),
            category_form_queryset.active_translations().count()
        )

        for category in category_form_queryset.all():
            # Each category string representation can be accessed without an error
            category_name = str(category)

            self.assertTrue(category_name)

    def test_create_alias_category_wizard_instance(self):
        wizard = self._get_wizard_instance('CreateAliasCategoryWizard')
        self.assertEqual(wizard.title, 'New alias category')

        self.assertTrue(wizard.user_has_add_permission(self.superuser))
        self.assertTrue(
            wizard.user_has_add_permission(self.get_staff_user_with_alias_permissions()),  # noqa: E501
        )
        self.assertFalse(
            wizard.user_has_add_permission(self.get_staff_user_with_no_permissions()),  # noqa: E501
        )
        self.assertFalse(
            wizard.user_has_add_permission(self.get_standard_user()),
        )

    def test_create_alias_category_wizard_form(self):
        wizard = self._get_wizard_instance('CreateAliasCategoryWizard')
        data = {
            'name': 'Category 1',
        }

        form_class = step2_form_factory(
            mixin_cls=WizardStep2BaseForm,
            entry_form_class=wizard.form,
        )
        form = form_class(**self._get_form_kwargs(data))

        self.assertTrue(form.is_valid())
        category = form.save()

        with self.login_user_context(self.superuser):
            response = self.client.get(category.get_absolute_url())
        self.assertContains(response, data['name'])



from django.urls import path


from . import constants, views  # isort:skip


urlpatterns = [
    path(
        'create-alias/',
        views.create_alias_view,
        name=constants.CREATE_ALIAS_URL_NAME,
    ),
    path(
        'aliases/<int:pk>/usage/',
        views.alias_usage_view,
        name=constants.USAGE_ALIAS_URL_NAME,
    ),
    path(
        'detach-alias/<int:plugin_pk>/',
        views.detach_alias_plugin_view,
        name=constants.DETACH_ALIAS_PLUGIN_URL_NAME,
    ),
    path(
        'delete-alias/<int:pk>/',
        views.delete_alias_view,
        name=constants.DELETE_ALIAS_URL_NAME,
    ),
    path(
        'select2/',
        views.AliasSelect2View.as_view(),
        name=constants.SELECT2_ALIAS_URL_NAME,
    ),
    path(
        'category-select2/',
        views.CategorySelect2View.as_view(),
        name=constants.CATEGORY_SELECT2_URL_NAME,
    ),


]



from unittest import skipUnless

from django.contrib import admin
from django.contrib.sites.models import Site

from cms.utils import get_current_site

from djangocms_alias.constants import (
    SITE_FILTER_NO_SITE_VALUE,
    SITE_FILTER_URL_PARAM,
)
from djangocms_alias.filters import CategoryFilter
from djangocms_alias.models import Alias as AliasModel, AliasContent, Category
from djangocms_alias.utils import is_versioning_enabled

from .base import BaseAliasPluginTestCase


class SiteFiltersTestCase(BaseAliasPluginTestCase):

    def test_site_filter(self):
        """
        When rendering aliascontent manager site filter changing the site
        should filter the results.
        """
        current_site = get_current_site()
        another_site = Site.objects.create(
            name="Other site",
            domain="othersite.here"
        )
        empty_site = Site.objects.create(
            name="Empty site",
            domain="emptysite.here"
        )
        category = Category.objects.create(name='Site Filter Category')
        current_site_alias = AliasModel.objects.create(
            category=category,
            site=current_site,
        )
        current_site_alias_content = AliasContent.objects.create(
            alias=current_site_alias,
            name="Current Site Alias Content",
            language="en",
        )
        another_site_alias = AliasModel.objects.create(
            category=category,
            site=another_site,
        )
        another_site_alias_content = AliasContent.objects.create(
            alias=another_site_alias,
            name="Another Site Alias Content",
            language="en",
        )
        no_site_alias = AliasModel.objects.create(
            category=category,
        )
        no_site_alias_content = AliasContent.objects.create(
            alias=no_site_alias,
            name="No Site Alias Content",
            language="en",
        )

        # If versioning is enabled be sure to create a version
        if is_versioning_enabled():
            from djangocms_versioning.models import Version

            Version.objects.create(content=current_site_alias_content, created_by=self.superuser)
            Version.objects.create(content=another_site_alias_content, created_by=self.superuser)
            Version.objects.create(content=no_site_alias_content, created_by=self.superuser)

        base_url = self.get_admin_url(AliasModel, "changelist")

        with self.login_user_context(self.superuser):
            # en is the default language configured for the site
            response_default = self.client.get(base_url)
            # filter by aliases with the current site
            response_current_site = self.client.get(f"{base_url}?{SITE_FILTER_URL_PARAM}={current_site.pk}")
            # filter by aliases with a different site set
            response_other_site = self.client.get(f"{base_url}?{SITE_FILTER_URL_PARAM}={another_site.pk}")
            # filter by aliases with an empty site set
            response_empty_site = self.client.get(f"{base_url}?{SITE_FILTER_URL_PARAM}={empty_site.pk}")
            # filter by aliases with no site set
            response_no_site = self.client.get(f"{base_url}?{SITE_FILTER_URL_PARAM}={SITE_FILTER_NO_SITE_VALUE}")

        # By default, all alias are shown
        self.assertEqual(
            set(response_default.context["cl"].queryset),
            {
                current_site_alias,
                another_site_alias,
                no_site_alias,
            }
        )
        # Only alias attached to the current site are shown when filtered by the current site
        self.assertEqual(
            set(response_current_site.context["cl"].queryset),
            {current_site_alias}
        )
        # Only alias attached to the current site are shown when filtered by another site
        self.assertEqual(
            set(response_other_site.context["cl"].queryset),
            {another_site_alias}
        )
        # Only alias attached to the current site are shown when filtered by no site
        self.assertEqual(
            set(response_no_site.context["cl"].queryset),
            {no_site_alias}
        )
        # No are shown when filtered by an empty site
        self.assertEqual(
            set(response_empty_site.context["cl"].queryset),
            set()
        )


class CatergoryFiltersTestCase(BaseAliasPluginTestCase):

    @skipUnless(not is_versioning_enabled(), 'Test only relevant when no versioning')
    def test_category_filter_no_verisoning(self):
        """
        When rendering aliascontent manager category filter, changing the category
        should filter the results.
        """
        category_one = Category.objects.create(name='one')
        alias_one = AliasModel.objects.create(
            category=category_one,
            position=0,
        )
        expected_category_one_content = AliasContent.objects.create(
            alias=alias_one,
            name="EN Alias Content one",
            language="en",
        )
        category_two = Category.objects.create(name='two')
        alias_two = AliasModel.objects.create(
            category=category_two,
            position=1,
        )
        expected_category_two_content = AliasContent.objects.create(
            alias=alias_two,
            name="EN Alias Content two",
            language="en",
        )
        base_url = self.get_admin_url(AliasContent, "changelist")

        with self.login_user_context(self.superuser):
            response_default = self.client.get(base_url)
            # category one should have a result
            category_one_filter_response = self.client.get(f"{base_url}?category={category_one.id}")
            # category two should have a result
            category_two_filter_response = self.client.get(f"{base_url}?category={category_two.id}")

        # By default all alias contents are shown
        self.assertEqual(
            set(response_default.context["cl"].queryset),
            {
                expected_category_one_content,
                expected_category_two_content,
            }
        )
        # show alias contents filter by category one
        self.assertEqual(
            set(category_one_filter_response.context["cl"].queryset),
            {expected_category_one_content}
        )
        # show alias contents filter by category two
        self.assertEqual(
            set(category_two_filter_response.context["cl"].queryset),
            {expected_category_two_content}
        )

    @skipUnless(is_versioning_enabled(), 'Test only relevant when versioning enabled')
    def test_category_filter_with_verisoning(self):
        """
        When rendering aliascontent manager category filter, changing the category
        should filter the results.
        """
        from djangocms_versioning.models import Version

        category_one = Category.objects.create(name='one')
        alias_one = AliasModel.objects.create(
            category=category_one,
            position=0,
        )
        expected_category_one_content = AliasContent.objects.create(
            alias=alias_one,
            name="EN Alias Content one",
            language="en",
        )
        Version.objects.create(content=expected_category_one_content, created_by=self.superuser)
        category_two = Category.objects.create(name='two')
        alias_two = AliasModel.objects.create(
            category=category_two,
            position=1,
        )
        expected_category_two_content = AliasContent.objects.create(
            alias=alias_two,
            name="EN Alias Content two",
            language="en",
        )
        Version.objects.create(content=expected_category_two_content, created_by=self.superuser)
        base_url = self.get_admin_url(AliasModel, "changelist")

        with self.login_user_context(self.superuser):
            response_default = self.client.get(base_url)
            # category one should have a result
            category_one_filter_response = self.client.get(f"{base_url}?category={category_one.id}")
            # categopry two should have a result
            category_two_filter_response = self.client.get(f"{base_url}?category={category_two.id}")

        # By default, all alias contents are shown
        self.assertEqual(
            set(response_default.context["cl"].queryset),
            {
                alias_one,
                alias_two,
            }
        )
        # show alias contents filter by category one
        self.assertEqual(
            set(category_one_filter_response.context["cl"].queryset),
            {alias_one}
        )
        # show alias contents filter by category two
        self.assertEqual(
            set(category_two_filter_response.context["cl"].queryset),
            {alias_two}
        )

    def test_category_filter_lookups_ordered_alphabetical(self):
        """
        Category filter lookup choices should be ordered in alphabetical order
        """
        category_one = Category.objects.create(name='b - category')
        alias_one = AliasModel.objects.create(
            category=category_one,
            position=0,
        )
        AliasContent.objects.create(
            alias=alias_one,
            name="EN Alias Content one",
            language="en",
        )
        category_two = Category.objects.create(name='a - category')
        alias_two = AliasModel.objects.create(
            category=category_two,
            position=1,
        )
        AliasContent.objects.create(
            alias=alias_two,
            name="EN Alias Content two",
            language="en",
        )

        version_admin = admin.site._registry[AliasModel]
        category_filter = CategoryFilter(None, {"category": [""]}, AliasModel, version_admin)
        # Get the first choice in the filter lookup object
        first_lookup_value = category_filter.lookup_choices[0][1]
        # Lookup value should match the category name linked to alias content
        self.assertEqual(
            first_lookup_value, category_two.name
        )
        self.assertNotEqual(
            first_lookup_value, category_one.name
        )

    def test_category_filter_lookup_should_only_show_aliases_linked_to_content(self):
        """
        Category not linked to content should not be listed in the category filter lookups
        """
        category_one = Category.objects.create(name='b - category')
        alias_one = AliasModel.objects.create(
            category=category_one,
            position=0,
        )
        AliasContent.objects.create(
            alias=alias_one,
            name="EN Alias Content one",
            language="en",
        )
        AliasModel.objects.create(
            category=category_one,
            position=1,
        )
        # Second category w/o alias assigned
        category_two = Category.objects.create(name='a - category')

        version_admin = admin.site._registry[AliasModel]
        category_filter = CategoryFilter(None, {"category": [""]}, AliasModel, version_admin)

        # Get the first choice in the filter lookup object
        first_lookup_value = category_filter.lookup_choices[0][1]
        # Lookup choices should only display the category linked to content
        self.assertEqual(len(category_filter.lookup_choices), 1)
        # Lookup value should match the category name linked to alias content
        self.assertEqual(
            first_lookup_value, category_one.name
        )
        # Category not linked to alias content should not be listed in the choices
        self.assertNotEqual(
            first_lookup_value, category_two.name
        )



from django.template.response import TemplateResponse


def render_alias_content(request, alias_content):
    template = 'djangocms_alias/alias_content_preview.html'
    context = {'alias_content': alias_content}
    return TemplateResponse(request, template, context)



# Generated by Django 2.1.1 on 2018-09-07 11:12

import cms.utils.i18n
from django.db import migrations, models
from django.conf import settings
import django.db.models.deletion
import parler.models

from djangocms_alias.models import get_templates, TEMPLATE_DEFAULT


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0034_remove_pagecontent_placeholders'),
    ]

    operations = [
        migrations.CreateModel(
            name='Alias',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('position', models.PositiveIntegerField(default=0, verbose_name='position')),
            ],
            options={
                'verbose_name': 'alias',
                'verbose_name_plural': 'aliases',
                'ordering': ['position'],
            },
        ),
        migrations.CreateModel(
            name='AliasContent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=120, verbose_name='name')),
                ('language', models.CharField(choices=settings.LANGUAGES, default=cms.utils.i18n.get_current_language, max_length=10)),
                ('alias', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='contents', to='djangocms_alias.Alias')),
            ],
            options={
                'verbose_name': 'alias content',
                'verbose_name_plural': 'alias contents',
            },
        ),
        migrations.CreateModel(
            name='AliasPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='djangocms_alias_aliasplugin', serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(choices=get_templates(), default=TEMPLATE_DEFAULT, max_length=255, verbose_name='template')),
                ('alias', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cms_plugins', to='djangocms_alias.Alias', verbose_name='alias')),
            ],
            options={
                'verbose_name': 'alias plugin model',
                'verbose_name_plural': 'alias plugin models',
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='CategoryTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=120, unique=True, verbose_name='name')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='djangocms_alias.Category')),
            ],
            options={
                'verbose_name': 'category Translation',
                'db_table': 'djangocms_alias_category_translation',
                'db_tablespace': '',
                'managed': True,
                'default_permissions': (),
            },
        ),
        migrations.AddField(
            model_name='alias',
            name='category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='aliases', to='djangocms_alias.Category', verbose_name='category'),
        ),
        migrations.AlterUniqueTogether(
            name='categorytranslation',
            unique_together={('language_code', 'master')},
        ),
    ]



from django.template import RequestContext
from django.utils.translation import gettext_lazy as _

from cms.toolbar.utils import get_toolbar_from_request

from djangocms_internalsearch.base import BaseSearchConfig
from djangocms_internalsearch.helpers import get_request, get_version_object
from haystack import indexes

from .models import AliasContent


def get_title(obj):
    return obj.result.title


get_title.short_description = _('Title')


def get_category(obj):
    return obj.result.category


get_category.short_description = _('Category')


def get_language(obj):
    return obj.result.language


get_language.short_description = _("Language")


def get_url(obj):
    return obj.result.url


get_url.short_description = _("URL")


def get_version_status(obj):
    return obj.result.version_status


get_version_status.short_description = _("Version status")


class AliasContentConfig(BaseSearchConfig):
    # indexes definition
    title = indexes.CharField(model_attr="name")
    category = indexes.CharField()
    language = indexes.CharField(model_attr="language")
    url = indexes.CharField()
    version_status = indexes.CharField()

    # admin setting
    list_display = [get_title, get_category, get_language, get_url, get_version_status]
    search_fields = ()
    list_filter = ()

    model = AliasContent

    def prepare_text(self, obj):
        request = get_request(obj.language)
        context = RequestContext(request)
        if 'request' not in context:
            context['request'] = request

        toolbar = get_toolbar_from_request(request)
        renderer = toolbar.get_content_renderer()
        source = obj.placeholder
        if not source:
            return
        content = renderer.render_placeholder(
            placeholder=source,
            context=context,
            editable=False,
        )
        return content

    def prepare_url(self, obj):
        return obj.get_absolute_url()

    def prepare_category(self, obj):
        obj.alias.category.set_current_language(obj.language)
        return obj.alias.category.name

    def prepare_version_status(self, obj):
        version_obj = get_version_object(obj)
        if not version_obj:
            return
        return version_obj.state



from django import forms
from django.contrib import admin
from django.http import (
    Http404,
    HttpRequest,
    HttpResponse,
    HttpResponseRedirect,
)
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from cms.admin.utils import GrouperModelAdmin
from cms.utils.permissions import get_model_permission_codename
from cms.utils.urlutils import admin_reverse

from parler.admin import TranslatableAdmin

from .cms_config import AliasCMSConfig
from .constants import (
    CHANGE_ALIAS_URL_NAME,
    DELETE_ALIAS_URL_NAME,
    LIST_ALIAS_URL_NAME,
    USAGE_ALIAS_URL_NAME,
)
from .filters import CategoryFilter, SiteFilter
from .forms import AliasGrouperAdminForm
from .models import Alias, AliasContent, Category
from .urls import urlpatterns
from .utils import (
    emit_content_change,
    emit_content_delete,
    is_versioning_enabled,
)


__all__ = [
    'AliasAdmin',
    'CategoryAdmin',
    'AliasContentAdmin',
]

alias_admin_classes = [GrouperModelAdmin]
alias_admin_list_display = ['content__name', 'category', 'admin_list_actions']
djangocms_versioning_enabled = AliasCMSConfig.djangocms_versioning_enabled

if djangocms_versioning_enabled:
    from djangocms_versioning.admin import (
        ExtendedGrouperVersionAdminMixin,
        StateIndicatorMixin,
    )
    from djangocms_versioning.models import Version

    alias_admin_classes.insert(0, ExtendedGrouperVersionAdminMixin)
    alias_admin_classes.insert(0, StateIndicatorMixin)
    alias_admin_list_display.insert(-1, "get_author")
    alias_admin_list_display.insert(-1, "get_modified_date")
    alias_admin_list_display.insert(-1, "state_indicator")


@admin.register(Category)
class CategoryAdmin(TranslatableAdmin):
    list_display = ['name']

    def save_model(self, request, obj, form, change):
        change = not obj._state.adding
        super().save_model(request, obj, form, change)
        if change:
            # Don't emit delete content because there is on_delete=PROTECT for
            # category FK on alias
            emit_content_change(
                AliasContent._base_manager.filter(alias__in=obj.aliases.all()),
                sender=self.model,
            )


@admin.register(Alias)
class AliasAdmin(*alias_admin_classes):
    list_display = alias_admin_list_display
    list_display_links = None
    list_filter = (SiteFilter, CategoryFilter,)
    fields = ('content__name', 'category', 'site', 'content__language')
    readonly_fields = ('static_code', )
    form = AliasGrouperAdminForm
    extra_grouping_fields = ("language",)
    EMPTY_CONTENT_VALUE = mark_safe(_("<i>Missing language</i>"))

    def get_urls(self) -> list:
        return urlpatterns + super().get_urls()

    def get_actions_list(self) -> list:
        """Add alias usage list actions"""
        return super().get_actions_list() + [self._get_alias_usage_link]

    def can_change_content(self, request: HttpRequest, content_obj: AliasContent) -> bool:
        """Returns True if user can change content_obj"""
        if content_obj and is_versioning_enabled():
            version = Version.objects.get_for_content(content_obj)
            return version.check_modify.as_bool(request.user)
        return True

    def has_delete_permission(self, request: HttpRequest, obj: Alias = None) -> bool:
        # Alias can be deleted by users who can add aliases,
        # if that alias is not referenced anywhere.
        if obj:
            if not obj.is_in_use:
                return request.user.has_perm(
                    get_model_permission_codename(self.model, 'add'),
                )
            return request.user.is_superuser
        return False

    def save_model(self, request: HttpRequest, obj: Alias, form: forms.Form, change: bool) -> None:
        super().save_model(request, obj, form, change)

        # Only emit content changes if Versioning is not installed because
        # Versioning emits its own signals for changes
        if not is_versioning_enabled():
            emit_content_change(
                AliasContent._base_manager.filter(alias=obj),
                sender=self.model,
            )

    def get_deleted_objects(self, objs, request: HttpRequest) -> tuple:
        deleted_objects, model_count, perms_needed, protected = super().get_deleted_objects(objs, request)
        # This is bad and I should feel bad.
        if 'placeholder' in perms_needed:
            perms_needed.remove('placeholder')
        return deleted_objects, model_count, perms_needed, protected

    def delete_model(self, request: HttpRequest, obj: Alias):
        super().delete_model(request, obj)

        # Only emit content changes if Versioning is not installed because
        # Versioning emits it' own signals for changes
        if not is_versioning_enabled():
            emit_content_delete(
                AliasContent._base_manager.filter(alias=obj),
                sender=self.model,
            )

    def _get_alias_usage_link(self, obj: Alias, request: HttpRequest, disabled: bool = False) -> str:
        url = admin_reverse(USAGE_ALIAS_URL_NAME, args=[obj.pk])
        return self.admin_action_button(url, "info", _("View usage"), disabled=disabled)

    def _get_alias_delete_link(self, obj: Alias, request: HttpRequest) -> str:
        url = admin_reverse(DELETE_ALIAS_URL_NAME, args=[obj.pk])
        return self.admin_action_button(url, "bin", _("Delete Alias"),
                                        disabled=not self.has_delete_permission(request, obj))


@admin.register(AliasContent)
class AliasContentAdmin(admin.ModelAdmin):
    # Disable dropdown actions
    actions = None
    change_form_template = "admin/djangocms_alias/aliascontent/change_form.html"

    def changelist_view(self, request: HttpRequest, extra_context: dict = None) -> HttpResponse:
        """Needed for the Alias Content Admin breadcrumbs"""
        return HttpResponseRedirect(admin_reverse(
            LIST_ALIAS_URL_NAME,
        ))

    def change_view(self, request: HttpRequest, object_id: int, form_url: str = '', extra_context: dict = None) -> HttpResponse:
        """Needed for the Alias Content Admin breadcrumbs"""
        obj = self.model.admin_manager.filter(pk=object_id).first()
        if not obj:
            raise Http404()
        return HttpResponseRedirect(admin_reverse(
            CHANGE_ALIAS_URL_NAME, args=(obj.alias_id,)
        ) + f"?language={obj.language}")

    def has_module_permission(self, request: HttpRequest) -> bool:
        """Hides admin class in admin site overview"""

        return False



from operator import attrgetter
from unittest import skipUnless
from urllib.parse import urlparse

from cms.api import add_plugin, create_title
from cms.utils import get_current_site
from cms.utils.plugins import downcast_plugins
from cms.utils.urlutils import admin_reverse

from djangocms_alias.cms_plugins import Alias
from djangocms_alias.constants import SELECT2_ALIAS_URL_NAME
from djangocms_alias.forms import AliasPluginForm, AliasSelectWidget
from djangocms_alias.utils import is_versioning_enabled

from .base import BaseAliasPluginTestCase


class AliasPluginTestCase(BaseAliasPluginTestCase):

    def test_extra_plugin_items_for_regular_plugins(self):
        extra_items = Alias.get_extra_plugin_menu_items(
            self.get_request(self.page.get_absolute_url()),
            self.plugin,
        )
        self.assertEqual(len(extra_items), 1)
        extra_item = extra_items[0]
        self.assertEqual(extra_item.name, 'Create Alias')
        self.assertEqual(extra_item.action, 'modal')
        parsed_url = urlparse(extra_item.url)
        self.assertEqual(parsed_url.path, self.get_create_alias_endpoint())
        self.assertIn(f'plugin={self.plugin.pk}', parsed_url.query)

    def test_extra_plugin_items_for_alias_plugins(self):
        alias = self._create_alias()

        placeholder = self.placeholder
        page_content = None
        if is_versioning_enabled():
            # Can only edit page/content that is in DRAFT
            page_content = create_title(self.language, 'Draft Page', self.page, created_by=self.superuser)
            placeholder = page_content.get_placeholders().get(slot='content')

        alias_plugin = alias.get_content(self.language).populate(
            replaced_placeholder=placeholder,
        )
        extra_items = Alias.get_extra_plugin_menu_items(
            self.get_page_request(page=self.page, obj=page_content, user=self.superuser),
            alias_plugin,
        )

        self.assertEqual(len(extra_items), 2)
        first, second = extra_items
        self.assertEqual(first.name, 'Edit Alias')
        self.assertEqual(first.url, alias.get_absolute_url())
        self.assertEqual(first.action, 'sideframe')

        self.assertEqual(second.name, 'Detach Alias')
        self.assertEqual(second.action, 'modal')
        self.assertEqual(
            second.url,
            self.get_detach_alias_plugin_endpoint(alias_plugin.pk),
        )

    def test_extra_plugin_items_for_placeholder(self):
        extra_items = Alias.get_extra_placeholder_menu_items(
            self.get_page_request(page=self.page, user=self.superuser),
            self.placeholder,
        )
        self.assertEqual(len(extra_items), 1)
        extra_item = extra_items[0]
        self.assertEqual(extra_item.name, 'Create Alias')
        self.assertEqual(extra_item.action, 'modal')
        parsed_url = urlparse(extra_item.url)
        self.assertEqual(parsed_url.path, self.get_create_alias_endpoint())
        self.assertIn(
            f'placeholder={self.placeholder.pk}',
            parsed_url.query,
        )

    @skipUnless(is_versioning_enabled(), 'Test only relevant for versioning')
    def test_extra_plugin_items_with_versioning_checks(self):
        alias = self._create_alias()
        alias_plugin = alias.get_content(self.language).populate(
            replaced_placeholder=self.placeholder,
        )
        try:
            _obj = self.page.get_title_obj()
        except AttributeError:
            _obj = self.page.get_content_obj()
        extra_items = Alias.get_extra_plugin_menu_items(
            self.get_page_request(page=self.page, obj=_obj, user=self.superuser),
            alias_plugin,
        )

        self.assertEqual(len(extra_items), 1)
        first = extra_items[0]
        # We cannot detach alias on undraft page
        self.assertEqual(first.name, 'Edit Alias')
        self.assertEqual(first.url, alias.get_absolute_url())

    def test_rendering_plugin_on_page(self):
        alias = self._create_alias(published=True)
        add_plugin(
            alias.get_placeholder(self.language),
            'TextPlugin',
            language=self.language,
            body='Content Alias 1234',
        )
        add_plugin(
            alias.get_placeholder(self.language),
            Alias,
            language=self.language,
            alias=alias,
        )
        add_plugin(
            self.placeholder,
            Alias,
            language=self.language,
            alias=alias,
        )

        with self.login_user_context(self.superuser):
            response = self.client.get(self.page.get_absolute_url(self.language))

        self.assertContains(response, 'Content Alias 1234')

    def test_detach_alias(self):
        alias = self._create_alias()
        alias_placeholder = alias.get_placeholder(self.language)
        add_plugin(
            alias_placeholder,
            'TextPlugin',
            language=self.language,
            body='test 1',
        )
        add_plugin(
            alias_placeholder,
            'TextPlugin',
            language=self.language,
            body='test 2',
        )
        plugins = self.placeholder.get_plugins()
        self.assertEqual(plugins.count(), 1)
        alias_plugin = add_plugin(
            self.placeholder,
            Alias,
            language=self.language,
            alias=alias,
        )
        add_plugin(
            alias_placeholder,
            'TextPlugin',
            language=self.language,
            body='test 3',
        )

        self.assertEqual(plugins.count(), 2)
        Alias.detach_alias_plugin(alias_plugin, self.language)
        self.assertEqual(plugins.count(), 4)

    def test_detach_alias_correct_position(self):
        alias = self._create_alias([])
        alias_placeholder = alias.get_placeholder(self.language)
        add_plugin(
            alias_placeholder,
            'TextPlugin',
            language=self.language,
            body='test 1',
        )
        add_plugin(
            alias_placeholder,
            'TextPlugin',
            language=self.language,
            body='test 2',
        )
        plugins = self.placeholder.get_plugins()
        self.assertEqual(plugins.count(), 1)
        alias_plugin = add_plugin(
            self.placeholder,
            Alias,
            language=self.language,
            alias=alias,
        )
        add_plugin(
            self.placeholder,
            'TextPlugin',
            language=self.language,
            body='test 3',
        )
        self.assertEqual(plugins.count(), 3)
        Alias.detach_alias_plugin(alias_plugin, self.language)
        self.assertEqual(plugins.count(), 4)

        ordered_plugins = sorted(
            downcast_plugins(plugins),
            key=attrgetter('position'),
        )
        self.assertEqual(
            [str(plugin) for plugin in ordered_plugins],
            ['test', 'test 1', 'test 2', 'test 3'],
        )

    def test_create_alias_with_default_render_template(self):
        alias = self._create_alias()
        add_plugin(
            self.placeholder,
            Alias,
            language=self.language,
            alias=alias,
        )
        self.assertEqual(alias.cms_plugins.first().template, 'default')

    def test_create_alias_with_custom_render_template(self):
        alias_template = 'custom_alias_template'
        alias = self._create_alias()
        add_plugin(
            self.placeholder,
            Alias,
            language=self.language,
            alias=alias,
            template=alias_template,
        )
        self.assertEqual(alias.cms_plugins.first().template, alias_template)

    def test_create_alias_plugin_form_initial_category(self):
        alias = self._create_alias(
            self.placeholder.get_plugins(),
        )
        alias_plugin = add_plugin(
            self.placeholder,
            Alias,
            language=self.language,
            alias=alias,
        )
        form = AliasPluginForm(instance=alias_plugin)
        self.assertEqual(form.fields['category'].initial, alias.category)

    def test_create_alias_plugin_form_empty_category(self):
        form = AliasPluginForm()
        self.assertEqual(form.fields['category'].initial, None)

    def test_alias_widget_attrs_include_select2_view_url(self):
        widget = AliasSelectWidget()
        attrs = widget.build_attrs({})
        self.assertIn('data-select2-url', attrs)
        self.assertEqual(
            attrs['data-select2-url'],
            admin_reverse(SELECT2_ALIAS_URL_NAME),
        )

    def test_create_alias_from_plugin_list(self):
        plugins = self.placeholder.get_plugins()
        alias = self._create_alias(plugins)
        self.assertEqual(
            plugins[0].plugin_type,
            alias.get_placeholder(self.language).get_plugins()[0].plugin_type,
        )
        self.assertEqual(
            plugins[0].get_bound_plugin().body,
            alias.get_placeholder(self.language).get_plugins()[0].get_bound_plugin().body,
        )

    def test_replace_plugin_with_alias(self):
        alias = self._create_alias([self.plugin])
        alias_content = alias.get_content(self.language)
        alias_plugin = alias_content.populate(
            replaced_plugin=self.plugin,
        )
        plugins = self.placeholder.get_plugins()
        self.assertNotIn(
            self.plugin,
            plugins,
        )
        self.assertEqual(plugins[0].get_bound_plugin(), alias_plugin)
        self.assertEqual(alias_content.placeholder.get_plugins()[0].get_bound_plugin().body, 'test')  # noqa: E501

    def test_replace_plugin_with_alias_correct_position(self):
        second_plugin = add_plugin(
            self.placeholder,
            'TextPlugin',
            language=self.language,
            body='test 2',
        )
        add_plugin(
            self.placeholder,
            'TextPlugin',
            language=self.language,
            body='test 3',
        )
        alias = self._create_alias()
        alias_plugin = alias.get_content(self.language).populate(replaced_plugin=second_plugin)
        plugins = self.placeholder.get_plugins()
        self.assertNotIn(
            self.plugin,
            plugins,
        )
        self.assertEqual(plugins[1].get_bound_plugin(), alias_plugin)

        ordered_plugins = sorted(
            downcast_plugins(plugins),
            key=attrgetter('position'),
        )

        self.assertEqual(
            [plugin.plugin_type for plugin in ordered_plugins],
            ['TextPlugin', 'Alias', 'TextPlugin'],
        )

    def test_replace_placeholder_content_with_alias(self):
        add_plugin(
            self.placeholder,
            'TextPlugin',
            language=self.language,
            body='test 2',
        )
        alias = self._create_alias()
        alias_content = alias.get_content(self.language)
        alias_content.populate(replaced_placeholder=self.placeholder)
        plugins = self.placeholder.get_plugins()

        self.assertEqual(plugins.count(), 1)
        self.assertEqual(alias_content.placeholder.get_plugins().count(), 2)
        self.assertEqual(
            alias_content.placeholder.get_plugins()[1].get_bound_plugin().body,
            'test 2',
        )

    def test_create_alias_plugin_form_initial_site(self):
        """
        By default the initial values should be set
        for the current site preselected and with no
        category set.
        """
        current_site = get_current_site()

        # Initially load the empty add form
        form = AliasPluginForm(data={})

        self.assertEqual(form.fields['site'].initial, current_site)
        self.assertEqual(form.fields['category'].initial, None)

    def test_change_alias_plugin_form_initial_site(self):
        """
        By default, the initial values should be set
        that are taken from the alias object currently selected
        """
        current_site = get_current_site()
        alias = self._create_alias(
            self.placeholder.get_plugins(),
        )
        alias_plugin = add_plugin(
            self.placeholder,
            Alias,
            language=self.language,
            alias=alias,
        )
        form = AliasPluginForm(instance=alias_plugin)

        self.assertNotEqual(form.fields['site'].initial, alias.site)
        self.assertEqual(form.fields['site'].initial, current_site)
        self.assertEqual(form.fields['category'].initial, alias.category)
        self.assertNotEqual(form.fields['category'].initial, None)



from django import forms
from django.contrib import admin
from django.contrib.admin.widgets import (
    AdminTextInputWidget,
    RelatedFieldWidgetWrapper,
)
from django.contrib.sites.models import Site
from django.db import transaction
from django.utils.translation import gettext_lazy as _

from cms.models import CMSPlugin, Placeholder
from cms.utils import get_current_site
from cms.utils.permissions import (
    get_model_permission_codename,
    has_plugin_permission,
)
from cms.utils.urlutils import admin_reverse

from parler.forms import TranslatableModelForm

from .constants import CATEGORY_SELECT2_URL_NAME, SELECT2_ALIAS_URL_NAME
from .models import (
    Alias,
    Alias as AliasModel,
    AliasContent,
    AliasPlugin,
    Category,
)
from .utils import emit_content_change, is_versioning_enabled


__all__ = [
    'AliasPluginForm',
    'BaseCreateAliasForm',
    'CreateAliasForm',
    'CreateAliasWizardForm',
    'CreateCategoryWizardForm',
]


def get_category_widget(formfield, user):
    dbfield = AliasModel._meta.get_field('category')
    return RelatedFieldWidgetWrapper(
        formfield.widget,
        dbfield.remote_field,
        admin_site=admin.site,
        can_add_related=user.has_perm(
            get_model_permission_codename(Category, 'add'),
        ),
        can_change_related=user.has_perm(
            get_model_permission_codename(Category, 'change'),
        ),
        can_delete_related=user.has_perm(
            get_model_permission_codename(Category, 'delete'),
        ),
    )


class BaseCreateAliasForm(forms.Form):
    plugin = forms.ModelChoiceField(
        queryset=CMSPlugin.objects.exclude(plugin_type='Alias'),
        required=False,
        widget=forms.HiddenInput(),
    )
    placeholder = forms.ModelChoiceField(
        queryset=Placeholder.objects.all(),
        required=False,
        widget=forms.HiddenInput(),
    )
    language = forms.CharField(widget=forms.HiddenInput())

    def clean(self):
        cleaned_data = super().clean()

        plugin = cleaned_data.get('plugin')
        placeholder = cleaned_data.get('placeholder')

        if not plugin and not placeholder:
            raise forms.ValidationError(
                _('A plugin or placeholder is required to create an alias.')
            )

        if plugin and placeholder:
            raise forms.ValidationError(
                _(
                    'An alias can only be created from a plugin or placeholder, '  # noqa: E501
                    'not both.'
                )
            )

        return cleaned_data


class CreateAliasForm(BaseCreateAliasForm):
    name = forms.CharField(required=True, widget=AdminTextInputWidget())
    site = forms.ModelChoiceField(
        queryset=Site.objects.all(),
        required=False,
    )
    category = forms.ModelChoiceField(
        queryset=Category.objects.all(),
        required=True,
    )
    replace = forms.BooleanField(
        label=_('Replace current plugin'),
        help_text=_('Replace current plugin with alias'),
        required=False,
    )

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user')

        super().__init__(*args, **kwargs)

        if not has_plugin_permission(self.user, 'Alias', 'add'):
            self.fields['replace'].widget = forms.HiddenInput()

        self.set_category_widget(self.user)
        self.fields["site"].initial = get_current_site()

    def clean(self):
        cleaned_data = super().clean()

        if AliasContent.objects.filter(
            name=cleaned_data.get('name'),
            language=cleaned_data.get('language'),
            alias__category=cleaned_data.get('category'),
        ).exists():
            raise forms.ValidationError(
                _('Alias with this Name and Category already exists.')
            )

        return cleaned_data

    def set_category_widget(self, user):
        formfield = self.fields['category']
        formfield.widget = get_category_widget(formfield, user)

    def get_plugins(self):
        plugin = self.cleaned_data.get('plugin')
        placeholder = self.cleaned_data.get('placeholder')

        if placeholder:
            plugins = placeholder.get_plugins(
                self.cleaned_data.get('language'),
            )
        else:
            plugins = [plugin] + list(plugin.get_descendants())
        return list(plugins)

    def save(self):
        alias = AliasModel.objects.create(
            category=self.cleaned_data.get('category'),
            site=self.cleaned_data.get('site'),
        )
        alias_content = AliasContent.objects.create(
            alias=alias,
            name=self.cleaned_data.get('name'),
            language=self.cleaned_data.get('language'),
        )
        if is_versioning_enabled():
            from djangocms_versioning.models import Version
            Version.objects.create(content=alias_content, created_by=self.user)
        if self.cleaned_data.get('replace'):
            placeholder = self.cleaned_data.get('placeholder')
            plugin = self.cleaned_data.get('plugin')
            source_plugins = None
        else:
            placeholder, plugin = None, None
            source_plugins = self.get_plugins()
        new_plugin = alias_content.populate(
            replaced_placeholder=placeholder,
            replaced_plugin=plugin,
            plugins=source_plugins,
        )
        return alias, alias_content, new_plugin


class CreateAliasWizardForm(forms.Form):
    name = forms.CharField(
        label=_('Name'),
        required=True,
        widget=AdminTextInputWidget()
    )
    site = forms.ModelChoiceField(
        queryset=Site.objects.all(),
        required=False,
    )
    category = forms.ModelChoiceField(
        queryset=Category.objects.all(),
        required=True,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not getattr(self, 'user', None):
            self.user = self._request.user
        self.set_category_widget(self.user)
        self.fields["site"].initial = get_current_site()

    def set_category_widget(self, user):
        formfield = self.fields['category']
        formfield.widget = get_category_widget(formfield, user)

    @transaction.atomic
    def save(self):
        alias = AliasModel.objects.create(
            category=self.cleaned_data.get('category'),
            site=self.cleaned_data.get('site'),
        )
        alias_content = AliasContent(
            alias=alias,
            name=self.cleaned_data.get('name'),
            language=self.language_code,
        )
        alias_content.save()  # Does not create a Version object

        if is_versioning_enabled():
            from djangocms_versioning.models import Version
            Version.objects.create(content=alias_content, created_by=self._request.user)

        emit_content_change([alias_content])
        return alias


class CreateCategoryWizardForm(TranslatableModelForm):

    class Meta:
        model = Category
        fields = [
            'name',
        ]


class Select2Mixin:

    class Media:
        css = {
            'all': ('cms/js/select2/select2.css', ),
        }
        js = (
            'admin/js/jquery.init.js',
            'cms/js/select2/select2.js',
            'djangocms_alias/js/dist/bundle.alias.create.min.js',
            'djangocms_alias/js/alias_plugin.js',
        )


class CategorySelectWidget(Select2Mixin, forms.TextInput):
    def get_url(self):
        return admin_reverse(CATEGORY_SELECT2_URL_NAME)

    def build_attrs(self, *args, **kwargs):
        attrs = super().build_attrs(*args, **kwargs)
        attrs.setdefault('data-select2-url', self.get_url())
        return attrs


class AliasSelectWidget(Select2Mixin, forms.TextInput):

    def get_url(self):
        return admin_reverse(SELECT2_ALIAS_URL_NAME)

    def build_attrs(self, *args, **kwargs):
        attrs = super().build_attrs(*args, **kwargs)
        attrs.setdefault('data-select2-url', self.get_url())
        return attrs


class AliasPluginForm(forms.ModelForm):
    site = forms.ModelChoiceField(
        label=_("Site"),
        queryset=Site.objects.all(),
        widget=forms.HiddenInput,
        required=False,
    )

    category = forms.ModelChoiceField(
        label=_('Category'),
        queryset=Category.objects.all(),
        widget=CategorySelectWidget(
            attrs={
                'data-placeholder': _('Select category to restrict the list of aliases below'),  # noqa: E501
            },
        ),
        empty_label='',
        required=False,
    )
    alias = forms.ModelChoiceField(
        label=_('Alias'),
        queryset=AliasModel.objects.all(),
        widget=AliasSelectWidget(
            attrs={
                'data-placeholder': _('Select an alias'),
            },
        ),
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._set_category_widget_value()

    def _set_category_widget_value(self):
        """
        When the user loads the form the site and category should be pre-selected
        """
        # If the form is changing an existing Alias
        # Be sure to show the values for an Alias
        if self.instance and self.instance.pk:
            self.fields['category'].initial = self.instance.alias.category
        # Otherwise this is creation
        # Set the site to the current site by default
        else:
            pass
        self.fields['site'].initial = get_current_site()

    class Meta:
        model = AliasPlugin
        fields = (
            'site',
            'category',
            'alias',
            'template',
        )


class AliasGrouperAdminForm(forms.ModelForm):
    class Meta:
        model = Alias
        fields = "__all__"

    def clean(self):
        cleaned_data = super().clean()
        if AliasContent.admin_manager.filter(
            name=cleaned_data.get('name'),
            language=cleaned_data.get('language'),
            alias__category=cleaned_data.get('category'),
        ).exists():
            raise forms.ValidationError(
                _('Alias with this Name and Category already exists.')
            )

        return cleaned_data



# Generated by Django 4.1.8 on 2023-07-25 15:47

from django.db import migrations, models
import django.db.models.deletion

class Migration(migrations.Migration):

    dependencies = [
        ('cms', '__first__'),
        ('djangocms_alias', '0002_auto_20200723_1424'),
    ]

    operations = [
        migrations.AlterField(
            model_name='aliasplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='%(app_label)s_%(class)s', serialize=False, to='cms.cmsplugin'),
        ),
        migrations.AlterField(
            model_name='categorytranslation',
            name='name',
            field=models.CharField(max_length=120, verbose_name='name'),
        ),
        migrations.AlterUniqueTogether(
            name='categorytranslation',
            unique_together={('language_code', 'master'), ('name', 'language_code')},
        ),
    ]



import json

from django.contrib import admin
from django.core.exceptions import PermissionDenied
from django.db.models import Q
from django.http import HttpResponse, HttpResponseBadRequest, JsonResponse
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from django.utils.translation import (
    get_language,
    get_language_from_request,
    gettext_lazy as _,
)
from django.views.generic import ListView

from cms.models import Page
from cms.toolbar.utils import get_plugin_toolbar_info, get_plugin_tree_as_json

from .cms_plugins import Alias
from .forms import BaseCreateAliasForm, CreateAliasForm
from .models import Alias as AliasModel, AliasPlugin, Category
from .utils import emit_content_change


JAVASCRIPT_SUCCESS_RESPONSE = """
    <div><div class="messagelist">
    <div class="success"></div>
    </div></div>
"""


def detach_alias_plugin_view(request, plugin_pk):
    if not request.user.is_staff:
        raise PermissionDenied

    instance = get_object_or_404(AliasPlugin, pk=plugin_pk)

    if request.method == 'GET':
        opts = Alias.model._meta
        context = {
            'has_change_permission': True,
            'opts': opts,
            'root_path': reverse('admin:index'),
            'is_popup': True,
            'app_label': opts.app_label,
            'object_name': _('Alias'),
            'object': instance.alias,
        }
        return render(request, 'djangocms_alias/detach_alias.html', context)

    language = get_language_from_request(request, check_path=True)

    plugins = instance.alias.get_plugins(language)

    can_detach = Alias.can_detach(request.user, instance.placeholder, plugins)

    if not can_detach:
        raise PermissionDenied

    copied_plugins = Alias.detach_alias_plugin(
        plugin=instance,
        language=language,
    )

    return render_replace_response(
        request,
        new_plugins=copied_plugins,
        source_plugin=instance,
    )


def delete_alias_view(request, pk, *args, **kwargs):
    from djangocms_alias.admin import AliasAdmin

    alias_admin = AliasAdmin(
        model=AliasModel,
        admin_site=admin.site,
    )
    response = alias_admin.delete_view(request, str(pk))
    if request.POST and response.status_code in [200, 302]:
        return HttpResponse(JAVASCRIPT_SUCCESS_RESPONSE)
    return response


def create_alias_view(request):
    if not request.user.is_staff:
        raise PermissionDenied

    form = BaseCreateAliasForm(request.GET or None)

    if form.is_valid():
        initial_data = form.cleaned_data
    else:
        initial_data = None

    if request.method == 'GET' and not form.is_valid():
        return HttpResponseBadRequest('Form received unexpected values')

    user = request.user

    create_form = CreateAliasForm(
        request.POST or None,
        initial=initial_data,
        user=user,
    )

    if not create_form.is_valid():
        opts = Alias.model._meta
        context = {
            'form': create_form,
            'has_change_permission': True,
            'opts': opts,
            'root_path': reverse('admin:index'),
            'is_popup': True,
            'app_label': opts.app_label,
            'media': (Alias().media + create_form.media),
        }
        return render(request, 'djangocms_alias/create_alias.html', context)

    plugins = create_form.get_plugins()

    if not plugins:
        return HttpResponseBadRequest(
            'Plugins are required to create an alias',
        )

    replace = create_form.cleaned_data.get('replace')
    if not Alias.can_create_alias(user, plugins, replace):
        raise PermissionDenied

    alias, alias_content, alias_plugin = create_form.save()
    emit_content_change([alias_content])

    if replace:
        plugin = create_form.cleaned_data.get('plugin')
        placeholder = create_form.cleaned_data.get('placeholder')
        return render_replace_response(
            request,
            new_plugins=[alias_plugin],
            source_placeholder=placeholder,
            source_plugin=plugin,
        )

    return HttpResponse(JAVASCRIPT_SUCCESS_RESPONSE)


def render_replace_response(request, new_plugins, source_placeholder=None,
                            source_plugin=None):
    move_plugins, add_plugins = [], []
    for plugin in new_plugins:
        root = plugin.parent.get_bound_plugin() if plugin.parent else plugin

        plugins = [root] + list(root.get_descendants())

        plugin_order = plugin.placeholder.get_plugin_tree_order(
            plugin.language,
            parent_id=plugin.parent_id,
        )
        plugin_tree = get_plugin_tree_as_json(request, plugins)
        move_data = get_plugin_toolbar_info(plugin)
        move_data['plugin_order'] = plugin_order
        move_data.update(json.loads(plugin_tree))
        move_plugins.append(json.dumps(move_data))
        add_plugins.append((
            json.dumps(get_plugin_toolbar_info(plugin)),
            plugin_tree,
        ))
    context = {
        'added_plugins': add_plugins,
        'moved_plugins': move_plugins,
        'is_popup': True,
    }
    if source_plugin is not None:
        context['replaced_plugin'] = json.dumps(
            get_plugin_toolbar_info(source_plugin),
        )
    if source_placeholder is not None:
        context['replaced_placeholder'] = json.dumps({
            'placeholder_id': source_placeholder.pk,
            'deleted': True,
        })
    return render(request, 'djangocms_alias/alias_replace.html', context)


class CategorySelect2View(ListView):
    queryset = Category.objects.order_by('translations__name')

    def get(self, request, *args, **kwargs):
        self.object_list = self.get_queryset()
        context = self.get_context_data()
        return JsonResponse({
            'results': [
                {
                    'text': str(obj),
                    'id': obj.pk,
                }
                for obj in context['object_list']
            ],
            'more': context['page_obj'].has_next(),
        })

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            raise PermissionDenied
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        """
        Only show Categories that have an Alias attached.
        If site is selected, use that to filter further.
        """
        term = self.request.GET.get('term')
        site = self.request.GET.get('site')
        queryset = super().get_queryset()
        # Only get categories that have aliases attached
        queryset = queryset.filter(
            aliases__isnull=False
        )

        try:
            pk = int(self.request.GET.get('pk'))
        except (TypeError, ValueError):
            pk = None

        q = Q()
        if term:
            q &= Q(translations__name__icontains=term)
        if site:
            q &= Q(aliases__site=site) | Q(aliases__site=None)
        if pk:
            q &= Q(pk=pk)

        return queryset.translated(get_language()).filter(q).distinct()

    def get_paginate_by(self, queryset):
        return self.request.GET.get('limit', 30)


class AliasSelect2View(ListView):
    queryset = AliasModel.objects.order_by('category__translations__name', 'position')

    def get(self, request, *args, **kwargs):
        self.object_list = self.get_queryset()
        context = self.get_context_data()
        return JsonResponse({
            'results': [
                {
                    'text': str(obj),
                    'id': obj.pk,
                }
                for obj in context['object_list']
            ],
            'more': context['page_obj'].has_next(),
        })

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            raise PermissionDenied
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        term = self.request.GET.get('term')
        category = self.request.GET.get('category')
        site = self.request.GET.get('site')
        # Showing published and unpublished aliases
        queryset = super().get_queryset().filter(
            contents__language=get_language(),
        ).distinct()

        try:
            pk = int(self.request.GET.get('pk'))
        except (TypeError, ValueError):
            pk = None

        q = Q()
        if term:
            q &= Q(contents__name__icontains=term)
        if category:
            q &= Q(category=category)
        if site:
            q &= Q(site=site) | Q(site=None)
        if pk:
            q &= Q(pk=pk)

        return queryset.filter(q).distinct()

    def get_paginate_by(self, queryset):
        return self.request.GET.get('limit', 30)


def alias_usage_view(request, pk):
    if not request.user.is_staff:
        raise PermissionDenied

    alias = get_object_or_404(AliasModel.objects.all(), pk=pk)
    opts = Alias.model._meta
    title = _(f'Objects using alias: {alias}')
    context = {
        'has_change_permission': True,
        'opts': opts,
        'root_path': reverse('admin:index'),
        'is_popup': True,
        'app_label': opts.app_label,
        'object_name': _('Alias'),
        'object': alias,
        'title': title,
        'original': title,
        'show_back_btn': request.GET.get('back'),
        'objects_list': sorted(
            alias.objects_using,
            # First show Pages on list
            key=lambda obj: isinstance(obj, Page),
            reverse=True,
        ),
    }
    return render(request, 'djangocms_alias/alias_usage.html', context)



from django.contrib import admin
from django.utils.encoding import smart_str
from django.utils.translation import get_language, gettext_lazy as _

from cms.forms.utils import get_sites

from .cms_config import AliasCMSConfig
from .constants import (
    CATEGORY_FILTER_PARAM,
    SITE_FILTER_NO_SITE_VALUE,
    SITE_FILTER_URL_PARAM,
)
from .models import Category


djangocms_versioning_enabled = AliasCMSConfig.djangocms_versioning_enabled


class SiteFilter(admin.SimpleListFilter):
    title = _("Site")
    parameter_name = SITE_FILTER_URL_PARAM

    def lookups(self, request, model_admin):
        return [(site.pk, site.name) for site in get_sites()]

    def queryset(self, request, queryset):
        chosen_site = self.value()
        if chosen_site and chosen_site == SITE_FILTER_NO_SITE_VALUE:
            return queryset.filter(site__isnull=True)
        elif chosen_site:
            return queryset.filter(site__pk=int(chosen_site))
        return queryset

    def choices(self, changelist):
        yield {
            "selected": self.value() is None,
            "query_string": changelist.get_query_string(remove=[self.parameter_name]),
            "display": _("All"),
        }
        yield {
            "selected": self.value() == SITE_FILTER_NO_SITE_VALUE,
            "query_string": changelist.get_query_string(
                {self.parameter_name: SITE_FILTER_NO_SITE_VALUE}
            ),
            "display": _("No site"),
        }
        for lookup, title in self.lookup_choices:
            yield {
                "selected": self.value() == str(lookup),
                "query_string": changelist.get_query_string(
                    {self.parameter_name: lookup}
                ),
                "display": title,
            }


class CategoryFilter(admin.SimpleListFilter):
    title = _("Category")
    parameter_name = CATEGORY_FILTER_PARAM

    def lookups(self, request, model_admin):
        # Only offer categories available
        qs = model_admin.get_queryset(request)
        cat_id = qs.values_list('category', flat=True)
        # Ensure the category is ordered by the name alphabetically by default
        cat = Category.objects.filter(pk__in=cat_id).translated(get_language()).order_by('translations__name')
        for obj in cat:
            yield str(obj.pk), smart_str(obj)

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(category=self.value()).distinct()

    def choices(self, changelist):
        yield {
            "selected": self.value() is None,
            "query_string": changelist.get_query_string(remove=[self.parameter_name]),
            "display": _("All"),
        }
        for lookup, title in self.lookup_choices:
            yield {
                "selected": self.value() == str(lookup),
                "query_string": changelist.get_query_string(
                    {self.parameter_name: lookup}
                ),
                "display": title,
            }



from django.utils.translation import gettext_lazy as _

from cms.utils.permissions import get_model_permission_codename
from cms.wizards.wizard_base import Wizard
from cms.wizards.wizard_pool import wizard_pool

from .cms_plugins import Alias
from .forms import CreateAliasWizardForm, CreateCategoryWizardForm
from .models import Alias as AliasModel, Category


class CreateAliasWizard(Wizard):

    def user_has_add_permission(self, user, **kwargs):
        return Alias.can_create_alias(user)


class CreateAliasCategoryWizard(Wizard):

    def user_has_add_permission(self, user, **kwargs):
        return user.has_perm(
            get_model_permission_codename(Category, 'add'),
        )


create_alias_wizard = CreateAliasWizard(
    title=_('New alias'),
    weight=200,
    form=CreateAliasWizardForm,
    model=AliasModel,
    description=_('Create a new alias.'),
)
create_alias_category_wizard = CreateAliasCategoryWizard(
    title=_('New alias category'),
    weight=200,
    form=CreateCategoryWizardForm,
    model=Category,
    description=_('Create a new alias category.'),
)

wizard_pool.register(create_alias_wizard)
wizard_pool.register(create_alias_category_wizard)



# Generated by Django 3.2.20 on 2023-07-26 13:43

from django.db import migrations, models
import django.utils.translation


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_alias', '0003_auto_20230725_1547'),
    ]

    operations = [
        migrations.AlterField(
            model_name='aliascontent',
            name='language',
            field=models.CharField(default=django.utils.translation.get_language, max_length=10),
        ),
    ]



from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType

from cms.api import add_plugin

from djangocms_alias.cms_plugins import Alias
from djangocms_alias.models import Alias as AliasModel
from djangocms_alias.utils import is_versioning_enabled

from .base import BaseAliasPluginTestCase


class AliasPermissionsTestCase(BaseAliasPluginTestCase):

    def test_can_create_alias_superuser(self):
        self.assertTrue(
            Alias.can_create_alias(
                self.get_superuser(),
                [self.plugin],
            ),
        )

    def test_can_create_alias_standard_user(self):
        self.assertFalse(
            Alias.can_create_alias(
                self.get_standard_user(),
                [self.plugin],
            ),
        )

    def test_can_create_alias_staff_no_permissions(self):
        self.assertFalse(
            Alias.can_create_alias(
                self.get_staff_user_with_no_permissions(),
                [self.plugin],
            ),
        )

    def test_can_create_alias_staff_partial_permissions(self):
        user = self.get_staff_user_with_no_permissions()
        user.user_permissions.add(
            Permission.objects.get(
                content_type=ContentType.objects.get_for_model(
                    AliasModel,
                ),
                codename='add_alias',
            )
        )
        user.user_permissions.add(
            Permission.objects.get(
                content_type=ContentType.objects.get_for_model(
                    Alias.model,
                ),
                codename='add_aliasplugin',
            )
        )
        alias = self._create_alias(self.placeholder.get_plugins())
        add_plugin(
            self.placeholder,
            Alias,
            language=self.language,
            alias=alias,
        )
        self.assertFalse(
            Alias.can_create_alias(
                user,
                self.placeholder.get_plugins(),
            ),
        )

    def test_can_create_alias_staff_enough_permissions(self):
        user = self.get_staff_user_with_std_permissions()
        user.user_permissions.add(
            Permission.objects.get(
                content_type=ContentType.objects.get_for_model(
                    AliasModel,
                ),
                codename='add_alias',
            )
        )
        self.assertTrue(
            Alias.can_create_alias(
                user,
                self.placeholder.get_plugins(),
            ),
        )

    def test_can_detach_no_permission(self):
        user = self.get_staff_user_with_no_permissions()
        alias = self._create_alias(self.placeholder.get_plugins())
        self.assertFalse(
            Alias.can_detach(
                user,
                self.placeholder,
                alias.get_placeholder(self.language).get_plugins(),
            ),
        )

    def test_can_detach_has_permission(self):
        user = self.get_staff_user_with_std_permissions()
        alias = self._create_alias(self.placeholder.get_plugins())
        placeholder = self.placeholder
        if is_versioning_enabled():
            placeholder = self._get_draft_page_placeholder()
        self.assertTrue(
            Alias.can_detach(
                user,
                placeholder,
                alias.get_placeholder(self.language).get_plugins(),
            ),
        )



from unittest import skipUnless

from django.contrib.sites.models import Site
from django.test.utils import override_settings

from cms.api import add_plugin, create_page, create_page_content
from cms.toolbar.utils import get_object_edit_url, get_object_preview_url

from djangocms_alias.cms_plugins import Alias
from djangocms_alias.constants import DEFAULT_STATIC_ALIAS_CATEGORY_NAME
from djangocms_alias.models import Alias as AliasModel, Category
from djangocms_alias.utils import is_versioning_enabled

from .base import BaseAliasPluginTestCase


class AliasTemplateTagsTestCase(BaseAliasPluginTestCase):
    alias_template = """{% load djangocms_alias_tags %}{% render_alias plugin.alias %}"""  # noqa: E501

    def test_render_alias(self):
        alias = self._create_alias()
        alias_plugin = alias.get_content(self.language).populate(
            replaced_placeholder=self.placeholder,
        )

        output = self.render_template_obj(
            self.alias_template,
            {
                'plugin': alias_plugin,
            },
            self.get_request('/'),
        )
        self.assertEqual(output, 'test')

    def test_render_alias_includes_recursed_alias(self):
        alias = self._create_alias()
        alias_plugin = alias.get_content(self.language).populate(
            replaced_placeholder=self.placeholder,
        )
        add_plugin(
            alias.get_placeholder(self.language),
            Alias,
            language=self.language,
            alias=alias,
        )

        output = self.render_template_obj(
            self.alias_template,
            {
                'plugin': alias_plugin,
            },
            self.get_request('/'),
        )
        self.assertEqual(output, 'test')

    @skipUnless(is_versioning_enabled(), 'Test only relevant for versioning')
    def test_render_alias_dont_render_draft_aliases(self):
        alias = self._create_alias([self.plugin], published=False)
        alias_plugin = add_plugin(
            self.placeholder,
            Alias,
            language=self.language,
            alias=alias,
        )
        output = self.render_template_obj(
            self.alias_template,
            {'plugin': alias_plugin},
            self.get_request('/'),
        )
        self.assertEqual(output, '')

        self._publish(alias)
        alias.clear_cache()

        output = self.render_template_obj(
            self.alias_template,
            {'plugin': alias_plugin},
            self.get_request('/'),
        )
        self.assertEqual(output, 'test')


class AliasTemplateTagAliasPlaceholderTestCase(BaseAliasPluginTestCase):
    alias_template = """{% load djangocms_alias_tags %}{% static_alias "some_unique_id" %}"""  # noqa: E501

    def test_no_alias_rendered_when_no_alias_exists(self):
        alias = self._create_alias(static_code="")
        alias_plugin = alias.get_content(self.language).populate(
            replaced_placeholder=self.placeholder,
        )
        add_plugin(
            alias.get_placeholder(self.language),
            'TextPlugin',
            language=self.language,
            body='Content Alias 1234',
        )

        output = self.render_template_obj(
            self.alias_template,
            {
                'plugin': alias_plugin,
            },
            self.get_request('/'),
        )
        self.assertEqual(output, "")

    def test_alias_rendered_when_alias_with_identifier_exists(self):
        alias = self._create_alias(static_code="some_unique_id")
        alias_plugin = alias.get_content(self.language).populate(
            replaced_placeholder=self.placeholder,
        )
        add_plugin(
            alias.get_placeholder(self.language),
            'TextPlugin',
            language=self.language,
            body='Content Alias 1234',
        )

        output = self.render_template_obj(
            self.alias_template,
            {
                'plugin': alias_plugin,
            },
            self.get_request('/'),
        )

        self.assertEqual(output, "testContent Alias 1234")

    def test_alias_auto_creation_in_template(self):
        """
        When a template discovers a static code that doesn't exist:
            - A category is created if it doesn't exist
            - An alias is created if one doesn't exist that matches the static_code
            - The creation_method is recorded as created by a template
            - If versioning is enabled the tag is only created for a user that is logged in
        """
        alias_template = """{% load djangocms_alias_tags %}{% static_alias "category_unique_code" %}"""  # noqa: E501

        # No Alias or Category exist
        category = Category.objects.filter(translations__name=DEFAULT_STATIC_ALIAS_CATEGORY_NAME)
        alias = AliasModel.objects.filter(static_code="category_unique_code")

        self.assertEqual(category.count(), 0)
        self.assertEqual(alias.count(), 0)

        # If versioning is enabled the tag is only created for a user that is logged in
        if is_versioning_enabled():
            self.render_template_obj(alias_template, {}, self.get_request('/'))

            self.assertEqual(category.count(), 0)
            self.assertEqual(alias.count(), 0)

        with self.login_user_context(self.superuser):
            # A default category, and a new alias is created for the template tag
            self.render_template_obj(alias_template, {}, self.get_request('/'))

        category_result = category.first()
        alias_result = alias.first()

        self.assertEqual(category.count(), 1)
        self.assertEqual(category_result.name, DEFAULT_STATIC_ALIAS_CATEGORY_NAME)
        self.assertEqual(alias.count(), 1)
        self.assertEqual(alias_result.static_code, "category_unique_code")
        self.assertEqual(alias_result.creation_method, AliasModel.CREATION_BY_TEMPLATE)

    def test_alias_auto_creation_in_template_site_limited_alias(self):
        """
        When a template discovers a static code for a site and with no site with the same static_code
        entries are created for both scenarios
        """
        unlimited_template = """{% load djangocms_alias_tags %}{% static_alias "limited_alias_code" %}"""  # noqa: E501
        site_limited_template = """{% load djangocms_alias_tags %}{% static_alias "limited_alias_code" site %}"""  # noqa: E501
        site_id = 1

        with self.login_user_context(self.superuser):
            # A default category, and a new alias is created for the template tag
            self.render_template_obj(unlimited_template, {}, self.get_request('/'))
            self.render_template_obj(site_limited_template, {}, self.get_request('/'))

        alias = AliasModel.objects.filter(static_code="limited_alias_code")

        self.assertEqual(len(alias), 2)
        self.assertEqual(alias[0].static_code, "limited_alias_code")
        self.assertEqual(alias[0].site, None)
        self.assertEqual(alias[1].static_code, "limited_alias_code")
        self.assertEqual(alias[1].site.pk, site_id)

        # Render both templates again and be sure that the original tags are reused
        self.render_template_obj(unlimited_template, {}, self.get_request('/'))
        self.render_template_obj(site_limited_template, {}, self.get_request('/'))

        alias_requery = AliasModel.objects.filter(static_code="limited_alias_code")

        self.assertEqual(alias_requery.count(), 2)

    def test_site_limited_alias_displays_the_correct_contents(self):
        """
        The correct contents are shown when viewing the static alias in different sites.
        """
        unlimited_template = """{% load djangocms_alias_tags %}{% static_alias "site_limit_alias_code" %}"""  # noqa: E501
        site_limited_template = """{% load djangocms_alias_tags %}{% static_alias "site_limit_alias_code" site %}"""  # noqa: E501
        site1 = Site.objects.create(domain='site1.com', name='1')
        site2 = Site.objects.create(domain='site2.com', name='2')

        category = Category.objects.create(name=DEFAULT_STATIC_ALIAS_CATEGORY_NAME)

        unlimited_alias = self._create_alias(
            plugins=None, name='test alias', category=category, static_code="site_limit_alias_code", site=None)
        site_limited_alias = self._create_alias(
            plugins=None, name='test alias', category=category, static_code="site_limit_alias_code", site=site2)

        add_plugin(
            unlimited_alias.get_placeholder(self.language),
            'TextPlugin',
            language=self.language,
            body='unlimited text',
        )
        add_plugin(
            site_limited_alias.get_placeholder(self.language),
            'TextPlugin',
            language=self.language,
            body='site limited text',
        )

        # Should show the contents of the unlimited template
        with override_settings(SITE_ID=site1.pk):
            site1_unlimited_preview = self.render_template_obj(unlimited_template, {}, self.get_request('/'))
            site1_limited_preview = self.render_template_obj(site_limited_template, {}, self.get_request('/'))

        self.assertEqual(site1_unlimited_preview, "unlimited text")
        self.assertEqual(site1_limited_preview, "")

        # Should show the contents of the site limited template
        with override_settings(SITE_ID=site2.pk):
            site2_unlimited_preview = self.render_template_obj(unlimited_template, {}, self.get_request('/'))
            site2_limited_preview = self.render_template_obj(site_limited_template, {}, self.get_request('/'))

        self.assertEqual(site2_unlimited_preview, "unlimited text")
        self.assertEqual(site2_limited_preview, "site limited text")

    @skipUnless(is_versioning_enabled(), 'Test only relevant for versioning')
    def test_static_alias_shows_correct_content_for_versioning_states(self):
        """
        The correct contents are shown when viewing the static alias:
        - A draft page shows draft content
        - A published page shows published content or nothing at all
        """
        from djangocms_versioning.constants import PUBLISHED

        category = Category.objects.create(name=DEFAULT_STATIC_ALIAS_CATEGORY_NAME)
        alias = self._create_alias(
            plugins=None,
            name='test alias',
            category=category,
            published=True,
            static_code="template_example_global_alias_code"
        )
        add_plugin(
            alias.get_placeholder(language='en'),
            'TextPlugin',
            language='en',
            body='Published content for: template_example_global_alias_code',
        )
        page = create_page(
            title="Static Code Test",
            language='en',
            template='static_alias.html',
            limit_visibility_in_menu=None,
            created_by=self.superuser
        )

        # Publish the page and create a draft alias
        self._publish(page, 'en')
        version = self._get_version(alias, PUBLISHED, 'en')
        draft = version.copy(self.superuser)

        # Add draft content to the draft version
        add_plugin(
            draft.content.placeholder,
            'TextPlugin',
            language='en',
            body='Updated Draft content for: template_example_global_alias_code',
        )
        try:
            page_content = page.get_title_obj("en")
        except AttributeError:
            page_content = page.get_content_obj("en")
        page_live_url = page.get_absolute_url()
        page_edit_url = get_object_edit_url(page_content, "en")
        page_preview_url = get_object_preview_url(page_content, "en")

        # The live page should still contain the published contents
        live_response = self.client.get(page_live_url)

        self.assertContains(live_response, 'Published content for: template_example_global_alias_code')
        self.assertNotContains(live_response, 'Updated Draft content for: template_example_global_alias_code')

        # The edit and preview url should show the draft contents
        with self.login_user_context(self.superuser):
            edit_response = self.client.get(page_edit_url, follow=True)
            preview_response = self.client.get(page_preview_url, follow=True)

        self.assertContains(edit_response, 'Updated Draft content for: template_example_global_alias_code')
        self.assertContains(preview_response, 'Updated Draft content for: template_example_global_alias_code')

    def test_static_alias_creates_content_for_missing_languages(self):
        """
        If a static alias is used by a logged-in user a first (empty) alias content object is created
        if no content objects for the language exist
        """

        page = create_page(
            title="Static Code Test",
            language='en',
            template='static_alias.html',
            limit_visibility_in_menu=None,
            created_by=self.superuser
        )
        create_page_content(
            title="Statischer Code-Test",
            language="de",
            page=page,
            template='static_alias.html',
            created_by=self.superuser
        )
        self._publish(page, "en")
        self._publish(page, "de")
        if hasattr(page, 'get_title_obj'):
            def page_edit_url(lang):
                return get_object_edit_url(page.get_title_obj(lang))
        else:
            def page_edit_url(lang):
                return get_object_edit_url(page.get_content_obj(lang), language=lang)

        with self.login_user_context(self.superuser):
            self.client.get(page_edit_url("en"), follow=True)  # supposed to create the alias and alias content for en
            self.client.get(page_edit_url("en"), follow=True)  # supposed to create no additional object
            self.client.get(page_edit_url("de"), follow=True)  # supposed to create the alias content for de

        alias = AliasModel.objects.filter(static_code="template_example_global_alias_code").first()

        self.assertIsNotNone(alias, "Alias not created")
        self.assertIsNotNone(alias.get_content("en", show_draft_content=True))
        self.assertIsNotNone(alias.get_content("de", show_draft_content=True))
        # Ensure that exactly two content objects have been created
        self.assertEqual(alias.contents(manager="admin_manager").count(), 2)

    def test_alias_rendered_when_identifier_is_variable(self):
        alias_template = """{% load djangocms_alias_tags %}{% static_alias foo_variable %}"""  # noqa: E501

        alias = self._create_alias(static_code="some_unique_id")
        add_plugin(
            alias.get_placeholder(self.language),
            'TextPlugin',
            language=self.language,
            body='Content Alias 1234',
        )

        output = self.render_template_obj(
            alias_template,
            {'foo_variable': "some_unique_id"},
            self.get_request('/'),
        )

        self.assertEqual(output, "Content Alias 1234")



PLUGIN_URL_NAME_PREFIX = 'djangocms_alias'

# Alias Url endpoint names
CREATE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_create'
DELETE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_delete'
DETACH_ALIAS_PLUGIN_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_detach_plugin'
CATEGORY_LIST_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_category_changelist'
CHANGE_CATEGORY_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_category_change'
SELECT2_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_select2'
USAGE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_alias_usage'
LIST_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_alias_changelist'
CHANGE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_alias_change'
DELETE_ALIAS_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_alias_delete'
CATEGORY_SELECT2_URL_NAME = f'{PLUGIN_URL_NAME_PREFIX}_category_list_select2'
# Static Alias
DEFAULT_STATIC_ALIAS_CATEGORY_NAME = "Static Alias"

# Admin Filters
SITE_FILTER_URL_PARAM = "site"
SITE_FILTER_NO_SITE_VALUE = "no_site"
UNPUBLISHED_FILTER_URL_PARAM = "unpublished"
CATEGORY_FILTER_PARAM = "category"



from djangocms_alias.utils import is_versioning_enabled

from .base import BaseAliasPluginTestCase


class AliasMenuTestCase(BaseAliasPluginTestCase):

    def test_alias_pages_have_no_menu_nodes(self):
        alias = self._create_alias()
        with self.login_user_context(self.superuser):
            response = self.client.get(alias.get_absolute_url())
            if is_versioning_enabled():
                self.assertNotContains(response, '<ul class="nav">')
            else:
                self.assertInHTML('<ul class="nav"></ul>', response.content.decode())

    def test_pages_keep_their_menu_nodes(self):
        """Tests that AliasDisableMenu modifier does not affect
        non-alias pages"""

        response = self.client.get(self.page.get_absolute_url())

        self.assertInHTML(
            '<ul class="nav"><li class="child selected"><a href="/en/test/">test</a></li></ul>',  # noqa: E501
            response.content.decode(),
        )



from copy import copy

from django.utils.translation import (
    get_language_from_request,
    gettext_lazy as _,
)

from cms.plugin_base import CMSPluginBase, PluginMenuItem
from cms.plugin_pool import plugin_pool
from cms.utils.permissions import (
    get_model_permission_codename,
    has_plugin_permission,
)
from cms.utils.plugins import copy_plugins_to_placeholder
from cms.utils.urlutils import add_url_parameters, admin_reverse

from .constants import CREATE_ALIAS_URL_NAME, DETACH_ALIAS_PLUGIN_URL_NAME
from .forms import AliasPluginForm
from .models import Alias as AliasModel, AliasContent, AliasPlugin


__all__ = [
    'Alias',
]


@plugin_pool.register_plugin
class Alias(CMSPluginBase):
    name = _('Alias')
    model = AliasPlugin
    form = AliasPluginForm

    def get_render_template(self, context, instance, placeholder):
        if (
            isinstance(instance.placeholder.source, AliasContent)
            and instance.is_recursive()
        ):
            return 'djangocms_alias/alias_recursive.html'
        return f'djangocms_alias/{instance.template}/alias.html'

    @classmethod
    def get_extra_plugin_menu_items(cls, request, plugin):
        if plugin.plugin_type == cls.__name__:
            edit_endpoint = plugin.alias.get_absolute_url()
            detach_endpoint = admin_reverse(
                DETACH_ALIAS_PLUGIN_URL_NAME,
                args=[plugin.pk],
            )

            plugin_menu_items = [
                PluginMenuItem(
                    _('Edit Alias'),
                    edit_endpoint,
                    action='sideframe',
                    attributes={'cms-icon': 'alias'},
                ),
            ]

            if cls.can_detach(
                request.user,
                plugin.placeholder,
                plugin.alias.get_plugins(),
            ):
                plugin_menu_items.append(
                    PluginMenuItem(
                        _('Detach Alias'),
                        detach_endpoint,
                        action='modal',
                        attributes={'cms-icon': 'alias'},
                    )
                )
            return plugin_menu_items

        data = {
            'plugin': plugin.pk,
            'language': get_language_from_request(request, check_path=True),
        }
        endpoint = add_url_parameters(admin_reverse(CREATE_ALIAS_URL_NAME), **data)
        return [
            PluginMenuItem(
                _('Create Alias'),
                endpoint,
                action='modal',
                attributes={'cms-icon': 'alias'},
            ),
        ]

    @classmethod
    def get_extra_placeholder_menu_items(cls, request, placeholder):
        data = {
            'placeholder': placeholder.pk,
            'language': get_language_from_request(request, check_path=True),
        }
        endpoint = add_url_parameters(admin_reverse(CREATE_ALIAS_URL_NAME), **data)

        menu_items = [
            PluginMenuItem(
                _('Create Alias'),
                endpoint,
                action='modal',
                attributes={'cms-icon': 'alias'},
            ),
        ]
        return menu_items

    @classmethod
    def can_create_alias(cls, user, plugins=None, replace=False):
        if not user.has_perm(
            get_model_permission_codename(AliasModel, 'add'),
        ):
            return False

        if not plugins:
            return True
        elif replace:
            target_placeholder = plugins[0].placeholder
            if (
                not target_placeholder.check_source(user)
                or not has_plugin_permission(user, Alias.__name__, 'add')
            ):
                return False

        return all(
            has_plugin_permission(
                user,
                plugin.plugin_type,
                'add',
            ) for plugin in plugins
        )

    @classmethod
    def can_detach(cls, user, target_placeholder, plugins):
        return all(
            has_plugin_permission(
                user,
                plugin.plugin_type,
                'add',
            ) for plugin in plugins
        ) and target_placeholder.check_source(user)

    @classmethod
    def detach_alias_plugin(cls, plugin, language):
        source_placeholder = plugin.alias.get_placeholder(language, show_draft_content=True)  # We're in edit mode
        target_placeholder = plugin.placeholder

        # Deleting uses a copy of a plugin to preserve pk on existing
        # ``plugin`` object. This is done due to
        # plugin.get_plugin_toolbar_info requiring a PK in a passed
        # instance.
        target_placeholder.delete_plugin(copy(plugin))
        target_placeholder._shift_plugin_positions(
            language,
            plugin.position,
            offset=target_placeholder.get_last_plugin_position(language),
        )
        if source_placeholder:
            source_plugins = source_placeholder.get_plugins_list()
            copied_plugins = copy_plugins_to_placeholder(
                source_plugins,
                placeholder=target_placeholder,
                language=language,
                start_positions={language: plugin.position},
            )
            return copied_plugins
        return []



from django.apps import apps


def is_versioning_enabled():
    from .models import AliasContent
    try:
        app_config = apps.get_app_config('djangocms_versioning')
        return app_config.cms_extension.is_content_model_versioned(AliasContent)
    except LookupError:
        return False


def emit_content_change(objs, sender=None):
    try:
        from djangocms_internalsearch.helpers import emit_content_change
    except ImportError:
        return

    for obj in objs:
        emit_content_change(obj, sender=sender)


def emit_content_delete(objs, sender=None):
    try:
        from djangocms_internalsearch.helpers import emit_content_delete
    except ImportError:
        return

    for obj in objs:
        emit_content_delete(obj, sender=sender)
